#
#   File        : pkgctree.py
#   Author      : Alain TAUCH <alain.tauch@thalesatm.com>
#   Description : build the ctree according to nodes
#                 designed to replace PkgTreeModel and TreeView
#
#   Copyright (C) 2003 Alain TAUCH
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import gobject
import gtk

class PkgCTree(gtk.CTree):
    
    def __init__(self, nodes, user = None, expanded = 0):
        gtk.CTree.__init__(self, 2, 0, ["packages tree", "version"])
        self.exp = expanded
        #self.build_it(nodes, user)
        self.build_rec(nodes, (0, ), None, user)
        self.user = None

    def set_style(self, node, idx, db):
        if db and db.has_key(idx):
            nfo = db[idx]
            if nfo[0] or nfo[1] or nfo[2]:
                self.node_set_foreground(node, gtk.gdk.color_parse("red"))
            elif nfo[3]:
                self.node_set_foreground(node, gtk.gdk.color_parse("blue"))
            else:
                self.node_set_foreground(node, gtk.gdk.color_parse("black"))    
    
    def build_rec(self, tree, idx, father, db):
        if tree.has_key(idx):
            nfo = tree[idx]
            nbchlds = len(nfo[1])
            me = self.insert_node(father, None, [nfo[0][0], nfo[0][1]],
                                  is_leaf = not nbchlds, expanded = self.exp)
            self.set_style(me, idx, db)
            for it in range(nbchlds):
                self.build_rec(tree, idx + (it, ), me, db)

    def build_it(self, tree, db):
        nodes = {None : [(0, ), ]}
        while nodes:
            newnodes = {}
            for it in nodes.keys():
                for it2 in nodes[it]:
                    nfo = tree[it2]
                    nbchlds = len(nfo[1])
                    me = self.insert_node(it, None, [nfo[0][0], nfo[0][1]],
                                          is_leaf = not nbchlds,
                                          expanded = self.exp)
                    self.set_style(me, it2, db)
                    subnodes = []
                    for it3 in range(nbchlds):
                        subnodes += [it2 + (it3, )]
                    if subnodes:
                        newnodes[me] = subnodes
            nodes = newnodes
