#!/usr/bin/python2
#
#   File        : rpm-analyzer.py
#   Author      : Alain TAUCH <alain.tauch@thalesatm.com>
#   Description : the RPMAnalyzer class
#
#   Copyright (C) 2003 Alain TAUCH
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   * Wed Aug 13 2003 Alain TAUCH <alain.tauch@thalesatm.com>
#   - added release and size info
#   * Mon Sep 15 2003 Laurent DENIEL <laurent.deniel@thalesatm.com>
#   - added command-line options
#   * Tue Feb 14 2006 Alain TAUCH <alain.tauch@thalesatm.com> 1.0r12
#   - support for python >= 2.2 (and RHEL4)
#   * Wed Feb 15 2006 Alain TAUCH <alain.tauch@thalesatm.com>
#   - improve robustness against broken comps                 1.0r13
#   - csv export                                              1.0r14
#   * Wed Feb 16 2006 Alain TAUCH <alain.tauch@thalesatm.com> 1.0r15
#   - fixed ScrollWindow+ViewPort+CTree unstable mix (SEGV when scrolling)
#   * Tue Feb 21 2006 Brian Long <brilong@cisco.com>          1.0r16
#   - set_text fix for pyGTK 2.x API
#   - keep track of comps user_visible flag
#   - keep track of optional packages
#   - display group name with group id in group tab
#   * Tue Feb 21 2006 Alain TAUCH <alain.tauch@thalesatm.com> 1.0r17
#   - coherent package display in group tab.
#   * Thu Jun 08 2006 Alain TAUCH <alain.tauch@thalesatm.com> 1.0r19
#   - case insensitive sorting option
#   * Tue Jun 13 2006 Alain TAUCH <alain.tauch@thalesatm.com> 1.21
#   - save/restore user preferences
#


import gtk
import gobject
import rpm
import rhpl.comps
from pkgctree import PkgCTree
import string
import debug
import sys
import getopt
import pango
import pickle
import cPickle
import os
import posixpath
from gtk import TRUE, FALSE
from types import *
# dm
from package_mgr.pm_rpm import RPMPackageManager, rpmvercmp

from winpref import WinPref
from winhtml import WinHtml
from wincsv  import WinCsv
from winchoice import WinChoice, CHOICE_SINGLE, CHOICE_MULTI
from winsearch import WinSearch

# profiler
#import profile, pstats

# debug level macros
DBG_ALL         = 100
DBG_NONE        = 0
DBG_ERROR       = 1
DBG_COMMON      = 10
DBG_LVL1        = 20
DBG_LVL2        = 30
DBG_LVL3        = 40
DBG_LVL4        = 50
DBG_LVL5        = 60
# manually set debug level here
DEBUG_LEVEL     = DBG_NONE

class   RPMAnalyzer:

    # pkglist's
    COLUMN_PKG_SEL      = 0
    COLUMN_PKG_ALL      = 1
    COLUMN_PKG_NAME     = 2
    COLUMN_PKG_VER      = 3
    COLUMN_PKG_RELEASE  = 4
    COLUMN_PKG_FSIZE    = 5    # file/rpm size
    COLUMN_PKG_ISIZE    = 6    # installed size
    # comps'
    COLUMN_COMPS_SEL    = 0
    COLUMN_COMPS_ALL    = 1
    COLUMN_COMPS_NAME   = 2
    # notebook
    NOTEBOOK_PAGE_GLOB  = 0
    NOTEBOOK_PAGE_DEP   = 1
    NOTEBOOK_PAGE_RDEP  = 2
    NOTEBOOK_PAGE_GROUP = 3
    # well, this is a dummy value
    # I do much prefer setting this to 1 than "dummy" to spare memory
    # and self.DUMMY seems more explicit than 1
    # I use a dictionnary (hashtable) to get unique values from a list
    # this is where dummy values are mainly used
    DUMMY       = 1
    # tooltips
    TOOLTIP_STATUS = """HDL: current hdlist
CMP: current comps.xml
DB: include current rpm db
PKG: selected/selected + deps/all (packages)
SIZE: selected size/total size"""
    TOOLTIP_DEP = "Dependency tree\nblue: already in tree\nred: already selected"
    TOOLTIP_RDEP = "Reverse Dependency tree\nblue: already in tree\n\
red: already selected"
    # backup
    BACKUP_VER  = 1
    VERSION     = "1.22"
    # local data
    LOCAL_HDL   = "/usr/share/comps/i386/hdlist"
    LOCAL_CMP   = "/usr/share/comps/i386/comps.xml"
    
    # MISC
    def huhu(self, a = 0, b = 0, c = 0, d = 0, e = 0):
        print "hu hu"

    def pref_sort(self, a, b):
        if self.winpref.case_sensitive_sort:
            return cmp(a,b)
        if type(a) is StringType or type(a) is UnicodeType:
            return cmp(a.lower(), b.lower())
        if type(a) is TupleType or type(a) is ListType:
            return cmp(a[0].lower(), b[0].lower())
        # fallback 
        self.d.info42(1, ("ERROR", "Unable to sort ", a), DBG_ERROR)
        return cmp(a,b)
    
    def quit(self, widget, user = None):
        self.d.info4(("naction quit", "", self, "", widget), DBG_COMMON)
        self.e_destroy(self.window)
        self.d.print_refcounts(60, DBG_LVL5)

    # when loading hdlist we clean everything
    # when loading comps we clean only comps related stuff
    def clean_members(self):
        # actually no more members to clean
        pass
        
    def clean_members_on_new_comps(self):
        self.clean_members()
        # file
        self.file_comps = ""
        # grp-pkg list
        self.compsl = {}
        # grp-grp list
        self.compsg = {}
        # pkg-grp list
        self.compsp = {}
        # deps cache
        self.dep_grppkg = {}
        self.dep_grpgrp = {}
        # selected groups
        self.comps_sel = {}
        # all selected groups and pkgs
        self.comps_grpinc = {}
        self.comps_pkginc = {}
        # cur group
        self.group = None
	# Group Display: grp-pkg mandatory list 
	self.compslm = {} 
	# Group Display: grp-pkg default list 
	self.compsld = {} 
	# Group Display: grp-pkg optional list 
	self.compslo = {} 
	# Group Display: grp-grp mandatory/default/optional list
	#  XXX: if needed we could to it too, see comment @update_group_info
	# Group Display: grp-id to name list
	self.grpname = {}
	# Group Display: user_visible
	self.user_visible = {}

    def clean_members_on_new_hdlist(self):
        # clean comps related members
        self.clean_members_on_new_comps()
        # clean rpmdb related members
        self.rpm_inc = {}

        # file
        self.file_hdlist = ""
        # all packages size
        self.size_all = 0
        # pkg list
        self.pkgl = {}
        # deps cache
        self.dep_pkgpkg = {}
        # tree cache
        self.dep_tree = {}
        self.rdep_tree = {}
        # selected pkgs
        self.pkg_sel = {}
        # all selected pkgs
        self.pkg_inc = {}
        # current pkg - list selection
        self.pkgnv = ()
        # current pkg - tree info
        self.dep_nodes = {}
        self.rdep_nodes = {}
        self.dep_nodes_u = {}
        self.rdep_nodes_u = {}
        # history
        self.history_idx = -1
        self.history = ()

    # force = 1  => rebuild all
    # force = 0 => refresh all
    # notebook = 1 => rebuild notebook
    # notebook = 0 => refresh notebook
    def refresh_all(self, force = 0, notebook = 0):
        self.refresh_pkglist(force)
        self.refresh_compslist(force)
        self.refresh_statusbar()
        self.refresh_notebook(force or notebook)

    # when we click on a checkbox, we don't want to select the item
    def toggle_check(self, path, user):
        if self.toggle == path:
            self.toggle = None
            return gtk.FALSE
        return gtk.TRUE

    def send_message(self, message):
        self.d.info4(("send message", "", message), DBG_COMMON)
        dialog = gtk.MessageDialog(self.window, gtk.DIALOG_DESTROY_WITH_PARENT,
                                   gtk.MESSAGE_WARNING, gtk.BUTTONS_CLOSE,
                                   message)
        dialog.connect('response', lambda dialog, response: dialog.destroy())
        dialog.show()

    def expand_tree(self, expanded):
        self.expand = expanded
        if self.show_dep:
            self.refresh_notebook_tree_dep()
        if self.show_rdep:
            self.refresh_notebook_tree_rdep()

    # groups and metapkg used the same way
    def include_grp_in_comps_deps(self, flag):
        if self.winpref.gm and flag == u'mandatory':
            return gtk.TRUE
        if self.winpref.gd and flag == u'default':
            return gtk.TRUE
        if self.winpref.go and flag == u'optional':
            return gtk.TRUE
        return gtk.FALSE

    def include_pkg_in_comps_deps(self, flag):
        if self.winpref.pm and flag == u'mandatory':
            return gtk.TRUE
        if self.winpref.pd and flag == u'default':
            return gtk.TRUE
        if self.winpref.po and flag == u'optional':
            return gtk.TRUE
        return gtk.FALSE
    #! MISC

    # CSV WINDOW RELATED
    def on_export_csv(self):
        self.export_csv(self.wincsv.file, self.wincsv.deps, self.wincsv.version,
                        self.wincsv.release, self.wincsv.fsize,
                        self.wincsv.isize, self.wincsv.description)

    #! CSV WINDOW RELATED

    # HTML WINDOW RELATED
    def on_export_html(self):
        self.export_status(self.winhtml.file, self.winhtml.css,
                           self.winhtml.deps, self.winhtml.sel_only,
                           self.winhtml.rel)
        
    #! HTML WINDOW RELATED

    # SEARCH WINDOW RELATED
    # XXX - should be ok    
    def on_search(self, str):
        self.d.info4(("Search requested", "string", str), DBG_COMMON)
        sel = self.pkg_tv.get_selection()
        (model, it) = sel.get_selected()
        if it:
            it = self.pkglist.iter_next(it)
        else:
            it = self.pkglist.get_iter_first()
        if self.winsearch.case == FALSE:
            str = string.lower(str)
        first = 1
        while it:
            name = self.pkglist.get_value(it, self.COLUMN_PKG_NAME)
            found = 0
            if self.winsearch.case == FALSE:
                name = string.lower(name)
            if self.winsearch.exact == TRUE:
                if name == str:
                    found = 1
            elif string.find(name, str) != -1:
                found = 1
            if found:
                path = self.pkglist.get_path(it)
                self.pkg_tv.scroll_to_cell(path,
                                           self.pkg_tv.get_column(0),
                                           gtk.TRUE,
                                           0.5,
                                           0)
                sel = self.pkg_tv.get_selection()
                sel.select_path(path)
                return
            it = self.pkglist.iter_next(it)
            if not it:
                if first:
                    first = 0
                    if self.ask("pattern not found, retry from the "
                                "beginning ?"):
                        it = self.pkglist.get_iter_first()
                else:
                    self.send_message("%s was not found in packages list"
                                      % (str, ))
    
    #! SEARCH WINDOW RELATED

    # PREF WINDOW RELATED
    # called when we made a preferences change
    def on_pref_change(self):
        if self.winpref.changed_comps:
            self.on_pref_change_comps()
        if self.winpref.changed_pkg:
            self.on_pref_change_pkg()
        # csv export change depending on pref too
        self.wincsv.set_pref(self.winpref.cv, self.winpref.cr,
                             self.winpref.cf, self.winpref.ci)
        # case (in)sensitive stuff
        if self.winpref.changed_case:
            # backup current selection
            pkgnv = self.pkgnv
            # refresh comps tree
            self.refresh_compslist(1)
            # refresh pkg tree
            self.refresh_pkglist(1)
            # restore current pkg selection
            if pkgnv:
                self.set_pkglist_sel(pkgnv)
            # refresh all tabs
            self.refresh_notebook()
        try:
            self.winpref.save_to_file(self.pref_file)
        except:
            self.d.info42(1, ("ERROR", "Unable to save preferences ", 
                              "file", self.pref_file), DBG_ERROR)
            

    def on_pref_change_pkg(self):
        # 
        tvc = self.pkg_tv.get_column(self.COLUMN_PKG_VER)
        if tvc.get_visible() != self.winpref.cv:
            tvc.set_visible(self.winpref.cv)
        tvc = self.pkg_tv.get_column(self.COLUMN_PKG_RELEASE)
        if tvc.get_visible() != self.winpref.cr:
            tvc.set_visible(self.winpref.cr)
        tvc = self.pkg_tv.get_column(self.COLUMN_PKG_FSIZE)
        if tvc.get_visible() != self.winpref.cf:
            tvc.set_visible(self.winpref.cf)
        tvc = self.pkg_tv.get_column(self.COLUMN_PKG_ISIZE)
        if tvc.get_visible() != self.winpref.ci:
            tvc.set_visible(self.winpref.ci)

    def on_pref_change_comps(self):
        # nothing appends if comps not loaded
        if not self.file_comps:
            return
        # backup the selection
        backup_sel = self.comps_sel
        # backup the comps filename
        backup_file = self.file_comps
        # same as import_comps, but we keep the selection
        # and update its deps
        self.clean_members_on_new_comps()
        self.build_comps_deps(backup_file)
        self.file_comps = backup_file
        self.comps_sel = backup_sel # XXX file may have changed
        self.rebuild_comps_pkginc()
        self.rebuild_comps_grpinc()
        self.refresh_all(force = 1)
    #! PREF WINDOW RELATED

    # TEXT TOOLS
    def create_text(self, is_source = gtk.FALSE):
        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC,
                                   gtk.POLICY_AUTOMATIC)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)
        text_view = gtk.TextView()
        scrolled_window.add(text_view)
        buffer = gtk.TextBuffer(None)
        text_view.set_buffer(buffer)
        text_view.set_editable(gtk.FALSE)
        text_view.set_cursor_visible(gtk.FALSE)
        text_view.set_wrap_mode(not is_source)
        # choose a fixed font for better display
        # Courier 9 is also good
        font_desc = pango.FontDescription("Monospace 9")
        text_view.modify_font(font_desc)
        return scrolled_window, buffer

    def sort_by_len(self, list):
        res = {}
        for it in list:
            size = len(it)
            if res.has_key(size):
                res[size] += [it, ]
            else:
                res[size] = [it, ]
        newlist = []
        for it in res.keys():
            newlist += res[it]
        return newlist

    # colwidth = 2 tabs with format_text
    # colwidth = 16 char with format_text2
    def format_list(self, list):
        if not list:
            return ""
        n = 0
        txt = ""
        colwidth = 16
        for it in list:
            data, nb = self.format_text2(it, colwidth)
            n += nb / colwidth
            if n >= 5:
                txt += "\n"
                n = nb / colwidth
            txt += data
        return txt
    
    # not used
    # require fixed font
    # better output when not using fixed fonts
    def format_text(self, txt, tabs):
        # 8 for shell
        tabsize = 8
        data = "%s" % (txt, )
        totalsize = ((len(data) / (tabs * tabsize)) + 1) * tabs * tabsize
        if ((totalsize - len(txt)) % tabsize) == 0:
            data += (((totalsize - len(txt)) / tabsize)) * '\t'
        else:
            data += (((totalsize - len(txt)) / tabsize) + 1) * '\t'
        return data, totalsize / tabsize

    # require fixed font
    def format_text2(self, txt, size):
        data = "%s" % (txt, )
        totalsize = ((len(data) / size) + 1) * size
        data += (totalsize - len(data)) * ' '
        return data, totalsize

    def set_empty(self, widget):
        txt = ""
        widget.set_text("")
    #! TEXT TOOLS

    # MESSAGE BOXES
    def ask(self, message):
        dialog = gtk.MessageDialog(self.window,
                                   gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                   gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, message)
        res = dialog.run()
        dialog.destroy()
        if res == gtk.RESPONSE_YES:
            return 1
        return 0
    #! MESSAGE BOXES

    # DEPENDENCY (LEVEL1) TREE TOOLS
    # tools related to the _global_ dependency tree
    # format: {pkgnv : [child1, child2, child3], ...}
    #
    # add pkg dep in deptree
    # returns new packages 
    def add_to_dep_tree(self, pkglist):
        self.d.info4(("add_to_dep_tree", pkglist), DBG_COMMON)
        newones = []
        for p in pkglist:
            if not self.dep_tree.has_key(p):
                realp = self.pkgl[p]
                reqlist = realp.requires
                self.solve_pkg_conflicts(realp, reqlist)
                reqlistnames = []
                for it in reqlist:
                    for it2 in it.relations:
                        name = it2.package.name
                        ver = it2.package.version 
                        if (name, ver) not in reqlistnames:
                            reqlistnames += [(name, ver)]
                        if (name, ver) not in newones:
                            newones += [(name, ver)]
                self.dep_tree[p] = reqlistnames
                self.d.info42(1, (p, reqlistnames), DBG_LVL1)
        return newones

    # used for deptree but not rdeptree
    def solve_pkg_conflicts(self, pkg, list):
        # if we include all packages, no need to go further
        if self.winpref.dep_behave == WinPref.BEHAVE_ALL:
            return
        self.d.info4(("solve pkg conflicts", "package", pkg.name), DBG_LVL1)
        for it in list:
            conflicts = []
            store = {}
            for it2 in it.relations:
                if it2.package.name not in conflicts:
                    conflicts += [it2.package.name, ]
                    store[it2.package.name] = it2
            if len(conflicts) > 1:
                self.d.info42(1, ("component", it), DBG_LVL1)
                # default: keep original info
                clean = it.relations
                # if the current pkg provides what it requires
                # let's forget about other packages
                # seems to be RH9-compliant
                if pkg.name in conflicts:
                    self.d.info42(2, (conflicts, "auto-solved"), DBG_LVL1)
                    clean = [store[pkg.name], ]
                else:
                    # ask user to solve the conflict
                    # should we set this to CHOICE_MULTI ?
                    if self.winpref.dep_behave == WinPref.BEHAVE_ASK:
                        ans = []
                        for it3 in conflicts:
                            ans += [(FALSE, "%s v.%s (%s KB)"
                                     % (it3, store[it3].package.version,
                                        store[it3].package.size / 1024)), ]
                        wc = WinChoice(CHOICE_SINGLE, "%s requires one of the "
                                       "following packages (%s):"
                                       % (pkg.name, it), ans,
                                       "packages relations conflict")
                        res = wc.run()
                        for it3 in range(len(res)):
                            if res[it3]:
                                clean = [store[conflicts[it3]], ]
                                break
                    # let's try to imitate RH9 behaviour
                    # refer to getfullcomps.py
                    elif self.winpref.dep_behave == WinPref.BEHAVE_RH9:
                        # sendmail is prefered over postfix
                        if "sendmail" in conflicts and "postfix" in conflicts:
                            clean = [store["sendmail"], ]
                        # shorter name wins
                        else:
                            winner = conflicts[0]
                            for it3 in conflicts:
                                if len(it3) < len(winner):
                                    winner = it3
                            clean = [store[winner], ]
                    self.d.info42(2, (conflicts, "-> [%s, ...] (total: %s)"
                                      % (clean[0].package.name, len(clean))),
                                  DBG_LVL1)
                it.relations = clean

    # add pkg dep in deptree
    # returns lower level pkg names and new packages 
    def add_to_rdep_tree(self, pkglist):
        self.d.info4(("add_to_rdep_tree", "", pkglist), DBG_COMMON)
        newones = []
        for p in pkglist:
            if not self.rdep_tree.has_key(p):
                realp = self.pkgl[p]
                reqlist = realp.provides
                reqlistnames = []
                for it in reqlist:
                    for it2 in it.relations:
                        name = it2.package.name
                        ver = it2.package.version 
                        if (name, ver) not in reqlistnames:
                            reqlistnames += [(name, ver)]
                        if (name, ver) not in newones:
                            newones += [(name, ver)]
                self.rdep_tree[p] = reqlistnames
                self.d.info42(1, (p, reqlistnames), DBG_LVL1)
        return newones

    def remove_from_dep_tree(self, pkglist):
        self.remove_from_tree(self.dep_tree, pkglist)
    
    def remove_from_rdep_tree(self, pkglist):
        self.remove_from_tree(self.rdep_tree, pkglist)
    
    def remove_from_tree(self, tree, pkglist):
        for p in pkglist:
            if tree.has_key(p):
                del tree[p]

    # Before displaying dep, we should make sure they were
    # analysed and prepared in dep_tree
    def build_full_dep(self, pkgnv):
        self.d.info4(("build_full_dep", "pkglist", pkgnv), DBG_COMMON)
        newpkg = [pkgnv]
        while len(newpkg) != 0:
            newpkg = self.add_to_dep_tree(newpkg)

    def build_full_rdep(self, pkgnv):
        self.d.info4(("build_full_rdep", "pkglist", pkgnv), DBG_COMMON)
        newpkg = [pkgnv]
        while len(newpkg) != 0:
            newpkg = self.add_to_rdep_tree(newpkg)

    def get_pkg_dep(self, pkg):
        if not self.dep_tree.has_key(pkg):
            self.build_full_dep(pkg)
        return self.dep_tree[pkg]

    def get_pkg_rdep(self, pkg):
        if not self.rdep_tree.has_key(pkg):
            self.build_full_rdep(pkg)
        return self.rdep_tree[pkg]
    #! DEPENDENCY (LEVEL1) TREE TOOLS

    # PKG DEPENDENCY (NODES) TREE TOOLS
    # tools related to the _selected package_ dependency tree
    #
    # depth = -1 for full deps tree
    #
    # nodes: {(0, ) : [pkgnv, [child1, child2, child3]],
    #         (0,0) : [pkgnv, [child1, child2, child3]],
    #         ... }
    #
    # newchilds: { (0,0) : [child1, child2], ... }
    #            { fathersnode : [packages_not_in_tree], ... }
    #
    def build_pkg_tree(self, pkgnv, tree, mindepth = 0, maxdepth = -1):
        self.d.info4(("build_pkg_tree", "pkg", pkgnv, "tree", tree, "mindepth",
                      mindepth, "maxdepth", maxdepth), DBG_COMMON)
        if not tree.has_key(pkgnv):
            self.d.info42(1, ("ERROR", "%s not in tree" % (pkgnv, )), DBG_ERROR)
            return
        nodes = {}
        nodes_info = {}
        curnode = ()
        newchilds = {}
        keys = {}
        childs = tree[pkgnv]
        newchilds[curnode] = [pkgnv]
        while newchilds:
            stop = len(curnode) >= maxdepth and maxdepth != -1
            go = mindepth > len(curnode) 
            otherchilds = {}
            self.d.info42(1, ("newchilds", newchilds), DBG_LVL4)
            for oldnode, it in newchilds.items():
                curpos = 0
                self.d.info42(2, ("childs", it), DBG_LVL4)
                for it2 in it:
                    curnode = oldnode + (curpos, )                
                    self.d.info42(3, ("node", "%s = %s" % (curnode, it2)),
                                  DBG_LVL4)
                    sel_pkg = self.pkg_inc and self.pkg_inc[it2]
                    sel_comp = (self.comps_pkginc or 0) and self.comps_pkginc[it2]
                    sel_db = (self.rpm_inc or 0) and self.rpm_inc[it2]
                    sel_prev = keys.has_key(it2)
                    # if pkg already selected
                    # or maxdepth reached
                    # and mindepth reached
                    if not go and \
                    (sel_pkg or sel_comp or sel_db or sel_prev or stop):
                        nodes[curnode] = [it2, []]
                    # if not already in tree
                    else:
                        childs = tree[it2]
                        if childs:
                            otherchilds[curnode] = childs
                        nodes[curnode] = [it2, childs]
                    keys[it2] = self.DUMMY
                    nodes_info[curnode] = (sel_pkg, sel_comp, sel_db, sel_prev)
                    curpos += 1
            newchilds = otherchilds
        return nodes, nodes_info

    def set_tree_depth_all(self, dummy, widget):
        self.set_tree_depth(-1)

    def set_tree_depth_one(self, dummy, widget):
        self.set_tree_depth(1)

    def set_tree_depth_two(self, dummy, widget):
        self.set_tree_depth(2)
        
    def set_tree_depth_three(self, dummy, widget):
        self.set_tree_depth(3)

    def set_tree_depth(self, depth):
        self.d.info4(("set_tree_depth", depth), DBG_COMMON)
        if not self.depth_tree == depth:
            self.depth_tree = depth
            self.refresh_notebook(1)
    
    # We don t want to rebuild it twice if not necessary
    #
    # we don't need to check depth since all is rebuild
    # when depth is changed
    def update_dep_nodes(self, pkgnv):
        if self.dep_nodes.has_key((0,)) and self.dep_nodes[(0,)][0] == pkgnv:
            return
        self.dep_nodes, self.dep_nodes_u  = self.build_pkg_tree(pkgnv,
                                                                self.dep_tree,
                                                                1,
                                                                self.depth_tree)

    def update_rdep_nodes(self, pkgnv):
        if self.rdep_nodes.has_key((0,)) and self.rdep_nodes[(0,)][0] == pkgnv:
            return
        self.rdep_nodes, self.rdep_nodes_u = self.build_pkg_tree(pkgnv,
                                                                 self.rdep_tree,
                                                                 1,
                                                                 self.depth_tree)
    #! PKG DEPENDENCY (NODES) TREE TOOLS 

    # COMPS DEPENDENCY (LEVEL1) TREE TOOLS
    #
    # self.compsl : {grpname : [pkg1, pkg2, ...], ...}
    #
    # self.compslm: {grpname : [pkg1, pkg2, ...], ...}
    # self.compsld: {grpname : [pkg1, pkg2, ...], ...}
    # self.compslo: {grpname : [pkg1, pkg2, ...], ...}
    #
    # self.compsg : {grpname : [grp1, grp2, ...], ...}
    #
    # self.compsp : {pkgnv : [grp1, grp2, ...], ...}
    #
    # /!\ call clean_members_on_new_comps before to init comps vars
    def build_comps_deps(self, compsfile):
        self.d.info4(("build_comps_deps", "file", compsfile), DBG_COMMON)
        try:
            comps = rhpl.comps.Comps(compsfile)
        except:
            self.d.info42(1, ("Error parsing comps.xml", sys.exc_info()),
                          DBG_ERROR)
            raise
        else:
            # all comps data cleaned before
            compsp = {}
            for group in comps.groups.values():
                self.d.info42(1, ("group", group.name, "id", group.id),
                              DBG_LVL1)
		self.d.info42(1, ("group", group.name, "id", group.id, "visible", 
                                  group.user_visible), DBG_LVL1)
		self.grpname[group.id] = group.name
                # fill self.compsg
                grps = []
                # we handle groups and metapkg the same way
                for (type, grp) in group.groups.values():
                    if self.include_grp_in_comps_deps(type):
                        grps += [grp]
                for (type, grp) in group.metapkgs.values():
                    if self.include_grp_in_comps_deps(type):
                        grps += [grp]
                self.d.info42(2, ("groups", grps), DBG_LVL1)
                self.compsg[group.id] = grps
                # fill self.compsl, self.compsl{m,d,o} and compsp
                pkgs = []
		pkgs_m = []
		pkgs_d = []
		pkgs_o = []
                for (type, pkg) in group.packages.values():
		    if type == u'mandatory':
			pkgs_m += [pkg]
		    elif type == u'default':
			pkgs_d += [pkg]
		    elif type == u'optional':
			pkgs_o += [pkg]
                    if self.include_pkg_in_comps_deps(type):
                        pkgs += [pkg]
                    if compsp.has_key(pkg):
                        compsp[pkg] += [group.id]
                    else:
                        compsp[pkg] = [group.id]
                self.d.info42(2, ("pkgs", pkgs), DBG_LVL1)
		self.d.info42(2, ("pkgs (mandatory)", pkgs_m), DBG_LVL1)
		self.d.info42(2, ("pkgs (default)", pkgs_d), DBG_LVL1)
		self.d.info42(2, ("pkgs (optional)", pkgs_o), DBG_LVL1)
                self.compsl[group.id] = pkgs
		self.compslm[group.id] = pkgs_m
		self.compsld[group.id] = pkgs_d
		self.compslo[group.id] = pkgs_o
                # fill self.comps_grpinc with default selected status
                self.comps_grpinc[group.id] = 0
                # fill self.comps_sel with default status
                self.comps_sel[group.id] = 0
		self.user_visible[group.id] = group.user_visible
            # fit compsp to current packages
            for it in self.pkgl.keys():
                if compsp.has_key(it[0]):
                    self.compsp[it] = compsp[it[0]]
                else:
                    self.compsp[it] = None
            self.d.info42(1, ("self.compsp", self.compsp), DBG_LVL1)
    #! COMPS DEPENDENCY (LEVEL1) TREE TOOLS

    # DATA IMPORT
    # use dm classes (pm_rpm in package_mgr)
    #
    # here we create the sel.pkgl
    # format: {(name, version) : package, ...}
    #
    # /!\ PLEASE NOTE /!\ hdlist2 is NOT hdlist
    def import_hdlist(self, hdl):
        self.d.info4(("import_hdlist", "name", hdl), DBG_COMMON)
        if not self.check_file(hdl, 'r'):
            self.send_message('hdlist not readable or not present')
            self.d.info42(1, ("hdlist not readable or not present", ),
                          DBG_ERROR)
            self.refresh_all(1)
            return
        # clean env each time we load a hdlist
        self.clean_members_on_new_hdlist()
        pm = RPMPackageManager()
        pkgs = None
        try:
            pkgs = pm.pkgset_from_hdlists((hdl, ))
        except:
            self.send_message('Error while loading hdlist')
            self.d.info42(1, ("Error parsing hdlist", sys.exc_info()),
                          DBG_ERROR)
            self.refresh_all(1)
        else:
            pkgs.resolve()
            self.file_hdlist = hdl
            self.d.info42(1, ("keys", pkgs), DBG_LVL1)
            # conflicts checking
            conflicts = {}
            # init self.pkgl and self.size_all
            for it in pkgs:
                ref = (it.name, it.version)
                # conflicts checking -> 1.5x slower (estim.)
                if conflicts.has_key(ref):
                    conflicts[ref] += [it, ]
                elif self.pkgl.has_key(ref):
                    conflicts[ref] = [self.pkgl[ref], it]
                # !conflicts
                self.pkgl[ref] = it
            self.solve_import_conflicts(conflicts)
            # /!\ len(pkgs) >= len(self.pkgl.keys())
            # init self.pkg_sel and self.pkg_inc
            for it in self.pkgl.keys():
                self.pkg_sel[it] = 0
                self.pkg_inc[it] = 0
                self.size_all += self.pkgl[it].size
            self.refresh_all(1)
            # activate some menubar options 
            self.menubar_item_set_sensitive("/Selection", gtk.TRUE)
            self.menubar_item_set_sensitive("/Export", gtk.TRUE)

    # /!\ comps.XML since RH 8.0
    # we also rebuild pkglist coz we may have selected some
    # groups before
    def import_comps(self, fname):
        self.d.info4(("import_comps", "name", fname), DBG_COMMON)
        if not self.check_file(fname, 'r'):
            self.send_message('comps.xml not readable or not present')
            self.d.info42(1, ("comps.xml not readable or not present", ),
                          DBG_ERROR)
            self.refresh_all(1)
            return
        if not self.file_hdlist:
            self.send_message('Load hdlist before comps.xml')
            return
        self.clean_members_on_new_comps()
        try:
            self.build_comps_deps(fname)
        except:
            self.send_message('Error while loading comps.xml')
            self.d.info42(1, ("Error parsing comps.xml", sys.exc_info()),
                          DBG_ERROR)
            self.refresh_all(1)
        else:
            self.file_comps = fname
            self.rebuild_comps_pkginc()
            self.refresh_compslist(force = 1)
            self.refresh_statusbar()

    # import packages from local db
    def import_db(self):
        if not self.file_hdlist:
            return
        pm = RPMPackageManager()
        pkgs = pm.pkgset_from_db()
        sel = {}
        for it in pkgs:
            sel[(it.name, it.version)] = self.DUMMY
        self.rpm_inc = {}
        for it in self.pkgl.keys():
            if sel.has_key(it):
                self.rpm_inc[it] = 1
            else:
                self.rpm_inc[it] = 0

    def open_file(self, winname, defaultpath, callback, user = None,
                  complete = None):
        self.d.info4(("open file", "winname", winname, "path", defaultpath,
                      "callback", callback, "user", user, "complete",
                      complete), DBG_COMMON)
        # tricky hack
        self.last_path = self.winhtml.last_path
        #
        if winname:
            filesel = gtk.FileSelection(winname)
        else:
            filesel = gtk.FileSelection("Select File")
        if defaultpath:
            filesel.set_filename(defaultpath)
        elif self.last_path:
            filesel.set_filename(self.last_path)
        if complete:
            filesel.complete(complete)
        result = filesel.run()
        if result == gtk.RESPONSE_OK:
            fname = filesel.get_filename()
            self.d.info42(1, ("fname", fname), DBG_LVL1)
            filesel.destroy()
            self.last_path = os.path.dirname(fname) + "/"
            # same here
            self.winhtml.last_path = self.last_path
            #
            if user:
                callback(fname, user)
            else:
                callback(fname)
        else:
            filesel.destroy()

    def open_backup_v1(self, upk):
        try:
            pkg_sel = upk.load()
            comps_sel = upk.load()
        except:
            self.send_message('Error while loading pkg and grp sel')
            self.d.info42(1, ("ERROR", sys.exc_info()), DBG_ERROR)
        else:
            # we must fit these selections to the current hdlist and comps
            self.pkg_sel = self.rebuild_base(pkg_sel, self.pkgl, self.no_dig)
            self.comps_sel = self.rebuild_base(comps_sel, self.compsl,
                                               self.no_dig)
            # then rebuild dependencies, which are not stored
            self.rebuild_pkg_inc()
            self.rebuild_comps_pkginc()
            self.rebuild_comps_grpinc()
            self.refresh_all()
    
    def open_pkg_and_grp_sel(self, fname):
        self.d.info4(("open pkg and group sel", "name", fname), DBG_COMMON)
        try:
            f = open(fname, 'r')
            upk = cPickle.Unpickler(f)
            ver = upk.load()
            if ver == 1:
                self.open_backup_v1(upk)
            else:
                self.send_message('backup version not supported')
            f.close()
        except:
            self.send_message('Error while loading pkg and grp sel')
            self.d.info42(1, ("ERROR", sys.exc_info()), DBG_ERROR)

    def check_file(self, file, mode):
        try:
            f = open(file, mode)
            f.close()
            return 1
        except:
            return 0

    def solve_import_conflicts(self, conflicts):
        """used by import_hdlist to solve conflicts between packages
        with same (name,ver) but different release or arch"""
        self.d.info42(1, ("conflicts", conflicts), DBG_COMMON)

        # well, where should I put this one ?
        toto = { WinPref.ROW_REL : self.keep_rel,
                 WinPref.ROW_386 : self.keep_386,
                 WinPref.ROW_586 : self.keep_586,
                 WinPref.ROW_686 : self.keep_686,
                 WinPref.ROW_ATH : self.keep_ath,
                 WinPref.ROW_ASK : self.keep_ask,
                 WinPref.ROW_NO : self.keep_no
                }
        
        for it in conflicts.keys():
            pool = conflicts[it]
            for i in range(len(self.winpref.hd_order)):
                newpool = toto[self.winpref.hd_order[i]](pool)
                if len(newpool) == 1:
                    self.pkgl[it] = newpool[0]
                    break
                elif len(newpool) > 1:
                    pool = newpool
            # if not enough, let's just ask
            else:
                self.pkgl[it] = toto[WinPref.ROW_ASK](pool)[0]
#        print "conflict resolution:"
#        for it in conflicts.keys():
#            a = self.pkgl[it]
#            print " - %s v.%s r.%s - %s" % (a.name, a.version, a.release,
#                                            a.arch)
#        print " "

    def keep_rel(self, pkgs):
        """take pkg list, return the newest ones (regarding release)"""
        max_ver = "0"
        new_pkgs = []
        for it in pkgs:
            cmp = rpmvercmp(max_ver, it.release)
            if not cmp:
                new_pkgs += [it, ]
            elif cmp == -1:
                max_ver = it.release
                new_pkgs = [it, ]
        return new_pkgs

    def keep_arch(self, pkgs, arch):
        """take pkg list, return the ones matching arch"""
        new_pkgs = []
        for i in pkgs:
            if i.arch == arch:
                new_pkgs += [i, ]
        return new_pkgs

    def keep_386(self, pkgs):
        return self.keep_arch(pkgs, "i386")
    
    def keep_586(self, pkgs):
        return self.keep_arch(pkgs, "i586")
    
    def keep_686(self, pkgs):
        return self.keep_arch(pkgs, "i686")

    def keep_ath(self, pkgs):
        return self.keep_arch(pkgs, "athlon")

    def keep_no(self, pkgs):
        """take pkg list, return the first pkg in list"""
        if len(pkgs):
            return [pkgs[0], ]
        return pkgs

    def keep_ask(self, pkgs):
        """take pkg list, return the choosen one (asking user for input)"""
        items = []
        for it in pkgs:
            items += [(FALSE, "%s v.%s r.%s - %s" % (it.name,
                                                     it.version,
                                                     it.release,
                                                     it.arch)), ]
        wc = WinChoice(CHOICE_SINGLE, "Choose package to use:",
                           items, "Import hdlist: package conflict")
        res = wc.run()
        for i in range(len(res)):
            if res[i]:
                return [pkgs[i], ]
        # should never come here, wc always return a result
        return []
                
    #! DATA IMPORT

    # DATA EXPORT
    # before calling update*nodes we must make sure all
    # requiered deps have been resolved
    def export_tree_dep(self, fname):
        self.d.info4(("export dep tree", "file", fname), DBG_COMMON)
        try:
            f = open(fname, 'w')
            self.build_full_dep(self.pkgnv)
            self.update_dep_nodes(self.pkgnv)
            txt = self.get_ascii_tree(self.dep_nodes, (0, ))
            f.write(txt)
            f.close()
        except:
            self.send_message('Error while saving dependency tree')
            self.d.info42(1, ("ERROR", sys.exc_info()), DBG_ERROR)
            
    def export_tree_rdep(self, fname):
        self.d.info4(("export rdep tree", "file", fname), DBG_COMMON)
        try:
            f = open(fname, 'w')
            self.build_full_rdep(self.pkgnv)
            self.update_rdep_nodes(self.pkgnv)
            txt = self.get_ascii_tree(self.rdep_nodes, (0, ))
            f.write(txt)
            f.close()
        except:
            self.send_message('Error while saving rev dependency tree')
            self.d.info42(1, ("ERROR", sys.exc_info()), DBG_ERROR)

    # for ascii export
    def get_ascii_tree(self, tree, node):
        data = ""
        if tree.has_key(node):
            nfo = tree[node]
            for it in node[:-1]:
                data += "\t"
            data += "- %s\n" % (nfo[0][0],)
            if len(nfo[1]):
                n = 0
                for it in nfo[1]:
                    data += self.get_ascii_tree(tree, node + (n,))
                    n += 1
        return data            

    # save selected packages and groups (col sel)
    # deps NOT included
    def save_pkg_and_grp_sel(self, fname):
        try:
            f = open(fname, 'w')
            pk = cPickle.Pickler(f, 1)
            pk.dump(self.BACKUP_VER)
            pk.dump(self.pkg_sel)
            pk.dump(self.comps_sel)
            f.close()
        except: 
            self.send_message('Error while saving pkg and grp sel')
            self.d.info42(1, ("ERROR", sys.exc_info()), DBG_ERROR)

    def export_csv(self, fname, deps, version, release, fsize, isize, descr):
        try:
            f = open(fname, "w")
            keyz = self.pkgl.keys()
            keyz.sort(self.pref_sort)
            for it in keyz:
                if deps:
                    pkg_sel = (self.pkg_inc or 0) and self.pkg_inc[it]
                    comps_sel = (self.comps_pkginc or 0) and self.comps_pkginc[it]
                    rpm_sel = (self.rpm_inc or 0) and self.rpm_inc[it]
                else:
                    pkg_sel = (self.pkg_sel or 0) and self.pkg_sel[it]
                    comps_sel = 0
                    rpm_sel = 0
                if pkg_sel or comps_sel or rpm_sel:
                    data = (it[0],)
                    format = '"%s"'
                    if version:
                        data += (self.pkgl[it].version,)
                        format += ';"%s"'
                    if release: 
                        data += (self.pkgl[it].release,)
                        format += ';"%s"'
                    if fsize:
                        data += (self.pkgl[it].fsize,)
                        format += ';"%d"'
                    if isize:
                        data += (self.pkgl[it].size,)
                        format += ';"%d"'
                    if descr:
                        data += (self.pkgl[it].summary,)
                        format += ';"%s"'
                    format += "\n"
                    f.write(format % data)
            f.close()
        except:
            self.send_message('Error while saving selection (csv)')
            self.d.info42(1, ("ERROR", sys.exc_info()), DBG_ERROR)

    # export selected packages
    def export_status(self, fname, css=0, deps=0, sel_only=0, rel=0):
        #
        refc = {"sel": ["sel", "#C0C0C0", "#FFFFFF"],
                "dep": ["dep", "#C0C0FF", "#FFFFFF"],
                "std": ["std", "#FFFFFF", "#000000"],
                "title": ["title", "#AB33BA", "#FFFFFF"]}
        refd = {"head": ["<HTML>\n<HEAD>\n"
                         "\t<TITLE>packages status</TITLE>\n"
                         "</HEAD>\n<BODY BACKGROUND=#FFFFFF>\n"
                         "<CENTER>\n<TABLE>\n",
                         "<HTML>\n<HEAD>\n"
                         "\t<TITLE>packages status</TITLE>\n"
                         "\t<STYLE TYPE=\"text/css\">\n"
                         "\t\tTABLE { border-collapse: collapse }\n"
                         "\t\t.title { background: @@TITBG@@ ;\n"
                         "\t\t         color: @@TITFG@@;\n"
                         "\t\t         border-bottom: solid #FFFFFF 3px;\n"
                         "\t\t         font-weight: bold; }\n"
                         "\t\t.sel { background-color: @@SELBG@@;\n"
                         "\t\t       color: @@SELFG@@; \n"
                         "\t\t       font-weight: bold; }\n"
                         "\t\t.dep { background-color: @@DEPBG@@;\n"
                         "\t\t       color: @@DEPFG@@; \n"
                         "\t\t       font-weight: bold; }\n"
                         "\t\t.std { background-color: @@STDBG@@;\n"
                         "\t\t       color: @@STDFG@@; \n"
                         "\t\t       font-weight: bold; }\n"
                         "\t\t.test { background: green }\n"
                         "\t\tTD { text-align: center }\n"
                         "\t\tTD:first-child { text-align: left }\n"
                         "\t</STYLE>\n"
                         "</HEAD>\n<BODY BACKGROUND=#FFFFFF>\n"
                         "<CENTER>\n<TABLE>\n"],
                "title": ["\t<TR BGCOLOR=@@BGCOLOR@@>\n"
                          "\t\t<TD><FONT COLOR=@@FGCOLOR@@>"
                          "<B>Package</B></FONT></TD>\n"
                          "\t\t<TD><FONT COLOR=@@FGCOLOR@@><B>Version</B>"
                          "</FONT></TD>\n\t\t<TD><FONT COLOR=@@FGCOLOR@@><B>"
                          "Summary</B></FONT></TD>\n\t</TR>\n",
                          "\t<TR CLASS=@@CLASS@@>\n"
                          "\t\t<TD>Package</TD>\n"
                          "\t\t<TD>Version</TD>\n"
                          "\t\t<TD>Summary</TD>\n"
                          "\t</TR>\n"],
                "data": ["\t<TR BGCOLOR=@@BGCOLOR@@>\n"
                         "\t\t<TD><FONT COLOR=@@FGCOLOR@@>%s</FONT></TD>\n"
                         "\t\t<TD><FONT COLOR=@@FGCOLOR@@>%s</FONT></TD>\n"
                         "\t\t<TD><FONT COLOR=@@FGCOLOR@@>%s</FONT></TD>\n"
                         "\t</TR>\n",
                         "\t<TR CLASS=@@CLASS@@>\n"
                         "\t\t<TD>%s</TD>\n"
                         "\t\t<TD>%s</TD>\n"
                         "\t\t<TD>%s</TD>\n"
                         "\t</TR>\n"],
                "foot": ["</CENTER>\n</TABLE>\n</BODY>\n</HTML>",
                         "</CENTER>\n</TABLE>\n</BODY>\n</HTML>"]}
        try:
            f = open(fname, "w")
            keyz = self.pkgl.keys()
            keyz.sort(self.pref_sort)
            if css:
                data = refd["head"][1]
                data = data.replace("@@TITBG@@", refc["title"][1])
                data = data.replace("@@TITFG@@", refc["title"][2])
                data = data.replace("@@SELBG@@", refc["sel"][1])
                data = data.replace("@@SELFG@@", refc["sel"][2])
                data = data.replace("@@DEPBG@@", refc["dep"][1])
                data = data.replace("@@DEPFG@@", refc["dep"][2])
                data = data.replace("@@STDBG@@", refc["std"][1])
                data = data.replace("@@STDFG@@", refc["std"][2])
            else:
                data = refd["head"][0]
            f.write(data)
            if css:
                data = refd["title"][1]
                data = data.replace("@@CLASS@@", refc["title"][0])
            else:
                data = refd["title"][0]
                data = data.replace("@@BGCOLOR@@", refc["title"][1])
                data = data.replace("@@FGCOLOR@@", refc["title"][2])
            f.write(data)
            for it in keyz:
                pkg_sel = self.pkg_sel[it]
                pkg_dep = (self.pkg_inc or 0) and self.pkg_inc[it]
                comps_dep = (self.comps_pkginc or 0) and self.comps_pkginc[it]
                rpm_dep = (self.rpm_inc or 0) and self.rpm_inc[it]
                if rel:
                    info = (it[0], it[1] + "-" + self.pkgl[it].release,
                            self.pkgl[it].summary)
                else:
                    info = it + (self.pkgl[it].summary, )
                if pkg_sel:
                    flag = "sel"
                elif pkg_dep or comps_dep or rpm_dep:
                    flag = "dep"
                else:
                    flag = "std"
                if flag == "dep":
                    if deps == self.winhtml.FL_DEPSASSEL:
                        flag = "sel"
                    elif deps == self.winhtml.FL_NODEPS:
                        flag = "std"
                if sel_only:
                    if flag == "std":
                        continue
                    else:
                        flag = "std"
                if css:
                    data = refd["data"][1]
                    data = data.replace("@@CLASS@@", refc[flag][0])
                else:
                    data = refd["data"][0]
                    data = data.replace("@@BGCOLOR@@", refc[flag][1])
                    data = data.replace("@@FGCOLOR@@", refc[flag][2])
                f.write(data % info)
            if css:
                data = refd["foot"][1]
            else:
                data = refd["foot"][0]
            f.write(data)
            f.close()
        except: 
            self.send_message('Error while saving html status')
            self.d.info42(1, ("ERROR", sys.exc_info()), DBG_ERROR)


    # export selected packages XXX deprecated
    def export_status_html(self, fname, css=0, deps=0):
        try:
            f = open(fname, "w")
            keyz = self.pkgl.keys()
            keyz.sort(self.pref_sort)
            f.write("<HTML>\n<HEAD>\n"
                    "\t<TITLE>packages status</TITLE>\n")
            if css:
                f.write("\t<STYLE TYPE=\"text/css\">\n"
                        "\t\tTABLE { border-collapse: collapse }\n"
                        "\t\t.title { background: #AB33BA ;\n"
                        "\t\t         color: #FFFFFF;\n"
                        "\t\t         border-bottom: solid #FFFFFF 3px;\n"
                        "\t\t         font-weight: bold; }\n"
                        "\t\t.sel { background-color: #C0C0C0;\n"
                        "\t\t       color: #FFFFFF; \n"
                        "\t\t       font-weight: bold; }\n"
                        "\t\t.test { background: green }\n"
                        "\t\tTD { text-align: center }\n"
                        "\t\tTD:first-child { text-align: left }\n"
                        "\t</STYLE>\n")
            f.write("</HEAD>\n<BODY>\n<CENTER>\n<TABLE>\n")
            if css:
                f.write("\t<TR CLASS=title>\n"
                        "\t\t<TD>Package</TD>\n"
                        "\t\t<TD>Version</TD>\n"
                        "\t\t<TD>Summary</TD>\n")
            else:
                f.write("\t<TR BGCOLOR=#AB33BA>\n"
                        "\t\t<TD><FONT COLOR=#FFFFFF><B>Package</B></FONT>"
                        "</TD>\n\t\t<TD><FONT COLOR=#FFFFFF><B>Version</B>"
                        "</FONT></TD>\n\t\t<TD><FONT COLOR=#FFFFFF><B>"
                        "Summary</B></FONT></TD>\n")
            f.write("\t</TR>\n")
            for it in keyz:
                if deps:
                    pkg_sel = (self.pkg_inc or 0) and self.pkg_inc[it]
                    comps_sel = (self.comps_pkginc or 0) and self.comps_pkginc[it]
                    rpm_sel = (self.rpm_inc or 0) and self.rpm_inc[it]
                else:
                    pkg_sel = (self.pkg_sel or 0) and self.pkg_sel[it]
                    comps_sel = 0
                    rpm_sel = 0
                data = it + (self.pkgl[it].summary, )
                if pkg_sel or comps_sel or rpm_sel:
                    if css:
                        f.write("\t<TR CLASS=sel>\n"
                                "\t\t<TD>%s</TD>\n"
                                "\t\t<TD>%s</TD>\n"
                                "\t\t<TD>%s</TD>\n" % data)
                    else:
                        f.write("\t<TR BGCOLOR=#C0C0C0>\n"
                                "\t\t<TD><FONT COLOR=#FFFFFF>%s</FONT></TD>\n"
                                "\t\t<TD><FONT COLOR=#FFFFFF>%s</FONT></TD>\n"
                                "\t\t<TD><FONT COLOR=#FFFFFF>%s</FONT></TD>\n"
                                % data)
                else:
                    f.write("\t<TR>\n"
                            "\t\t<TD>%s</TD>\n"
                            "\t\t<TD>%s</TD>\n"
                            "\t\t<TD>%s</TD>\n" % data)
                f.write("\t</TR>\n")
            f.write("</CENTER>\n</TABLE>\n</BODY>\n</HTML>")
            f.close()
        except: 
            self.send_message('Error while saving html status')
            self.d.info42(1, ("ERROR", sys.exc_info()), DBG_ERROR)

    # export all selected packages
    # tricky ask box here -> to be changed
    def export_all_sel(self, fname, with_ver = 0):
        deps = self.ask("Export deps as well ?")
        try:
            f = open(fname, "w")
            keyz = self.pkgl.keys()
            keyz.sort(self.pref_sort)
            for it in keyz:
                if deps == 1:
                    pkg_sel = (self.pkg_inc or 0) and self.pkg_inc[it]
                    comps_sel = (self.comps_pkginc or 0) and self.comps_pkginc[it]
                    rpm_sel = (self.rpm_inc or 0) and self.rpm_inc[it]
                else:
                    pkg_sel = (self.pkg_sel or 0) and self.pkg_sel[it]
                    comps_sel = 0
                    rpm_sel = 0
                if pkg_sel or comps_sel or rpm_sel:
                    if with_ver:
                        data = it + (self.pkgl[it].release, self.pkgl[it].summary)
                        f.write("%s-%s-%s %s\n" % data)
                    else:
                        f.write("%s\n" % (it[0]))
            f.close()
        except:
            self.send_message('Error while saving all sel')
            self.d.info42(1, ("ERROR", sys.exc_info()), DBG_ERROR)
    #! DATA EXPORT

    # PKGLIST AND COMPSLIST TOOLS
    # refresh pkglist, rebuild if force = 1
    def refresh_pkglist(self, force = 0):
        self.d.info4(("refresh_pkglist", "force", force), DBG_COMMON)
        if force:
            self.rebuild_pkglist()
        else:
            self.refresh_pkglist_sel()
    
    # Here we have to rebuild the pkglist using information from:
    # self.pkg_inc: user includes
    # self.pkg_sel: user selection
    # self.comps_pkginc: comps includes
    # self.rpm_inc: system rpms includes
    def rebuild_pkglist(self):
        list = self.pkgl.keys()
        list.sort(self.pref_sort)
        self.pkglist.clear()
        self.nb_sel = 0
        self.nb_all = 0
        self.size_sel = 0
        for it in list:
            it2 = self.pkglist.append()
            self.pkglist.set_value(it2, self.COLUMN_PKG_SEL, self.pkg_sel[it])
            self.pkglist.set_value(it2, self.COLUMN_PKG_NAME, it[0])
            self.pkglist.set_value(it2, self.COLUMN_PKG_VER, it[1])
            self.pkglist.set_value(it2, self.COLUMN_PKG_RELEASE,
                                   self.pkgl[it].release)
            self.pkglist.set_value(it2, self.COLUMN_PKG_FSIZE,
                                   self.pkgl[it].fsize / 1024)
            self.pkglist.set_value(it2, self.COLUMN_PKG_ISIZE,
                                   self.pkgl[it].size / 1024)
            # begin with user deps
            state = self.pkg_inc[it]
            # then comps deps if comps loaded
            if self.comps_pkginc:
                state = state or self.comps_pkginc[it]
            # then rpm db if needed
            if self.rpm_inc:
                state = state or self.rpm_inc[it]
            # then set the appropriate value
            self.pkglist.set_value(it2, self.COLUMN_PKG_ALL, state)
            # update self.nb_sel and self.nb_all and self.size
            self.nb_sel += self.pkg_sel[it]
            self.nb_all += state
            if state:
                self.size_sel += self.pkgl[it].size

    # here we update the selection cols
    def refresh_pkglist_sel(self):
        self.nb_sel = 0
        self.nb_all = 0
        self.size_sel = 0
        it = self.pkglist.get_iter_first()
        while it:
            pkg = (self.pkglist.get_value(it, self.COLUMN_PKG_NAME),
                   self.pkglist.get_value(it, self.COLUMN_PKG_VER))
            self.pkglist.set_value(it, self.COLUMN_PKG_SEL, self.pkg_sel[pkg])
            state = self.pkg_inc[pkg]
            if self.comps_pkginc:
                state = state or self.comps_pkginc[pkg]
            if self.rpm_inc:
                state = state or self.rpm_inc[pkg]
            self.pkglist.set_value(it, self.COLUMN_PKG_ALL, state)
            self.nb_sel += self.pkg_sel[pkg]
            self.nb_all += state
            if state:
                self.size_sel += self.pkgl[pkg].size
            it = self.pkglist.iter_next(it)

    # refresh compslist, rebuild if force = 1
    def refresh_compslist(self, force = 0):
        self.d.info4(("refresh_compslist", "force", force), DBG_COMMON)
        if force:
            self.rebuild_compslist()
        else:
            self.refresh_compslist_sel()

    # Here we rebuild all the compslist
    # using information stored in self.comps_sel and self.comps_grpinc
    def rebuild_compslist(self):
        list = self.comps_sel.keys()
        list.sort(self.pref_sort)
        self.compslist.clear()
        # display it
        for it in list:
            it2 = self.compslist.append()
            self.compslist.set_value(it2, self.COLUMN_COMPS_NAME, it)
            self.compslist.set_value(it2, self.COLUMN_COMPS_SEL,
                                     self.comps_sel[it])
            self.compslist.set_value(it2, self.COLUMN_COMPS_ALL,
                                     self.comps_grpinc[it])

    # here we update the selection cols
    def refresh_compslist_sel(self):
        it = self.compslist.get_iter_first()
        while it:
            grp = self.compslist.get_value(it, self.COLUMN_COMPS_NAME)
            self.compslist.set_value(it, self.COLUMN_COMPS_SEL,
                                     self.comps_sel[grp])
            self.compslist.set_value(it, self.COLUMN_COMPS_ALL,
                                     self.comps_grpinc[grp])
            it = self.compslist.iter_next(it)

    # for rebuild_base
    def no_dig(self, anything):
        return []
            
    # used by rebuild_*inc
    def rebuild_base(self, sel_list, ref_list, dig_func):
        sel = {}
        for it in sel_list.keys():
            if sel_list[it] and not sel.has_key(it):
                sel[it] = self.DUMMY
                for it2 in dig_func(it):
                    sel[it2] = self.DUMMY
        newone = {}
        for it in ref_list.keys():
            if sel.has_key(it):
                newone[it] = 1
            else:
                newone[it] = 0
        return newone

    # rebuild self.pkg_inc, regarding self.pkg_sel
    def rebuild_pkg_inc(self):
        self.pkg_inc = self.rebuild_base(self.pkg_sel,
                                         self.pkg_inc,
                                         self.get_pkgpkg_deps)
        
    # rebuild self.comps_grpinc, regarding self.comps_sel
    def rebuild_comps_grpinc(self):
        self.comps_grpinc = self.rebuild_base(self.comps_sel,
                                              self.comps_grpinc,
                                              self.get_grpgrp_deps)

    # rebuild self.comps_pkginc, regarding self.comps_sel
    # this part is using hdlist for pkg dependencies and
    # not dependencies stored in comps.xml.
    # We only use comps.xml to get the relation between groups
    # and packages.
    def rebuild_comps_pkginc(self):
        self.comps_pkginc = self.rebuild_base(self.comps_sel,
                                              self.pkgl,
                                              self.get_grppkg_deps)

    # change seleted item to pkgnv
    # used for CTRL+P CTRL+N and DBLE CLICK on tree
    # also used when redrawing after case sorting change
    def set_pkglist_sel(self, pkgnv):
        self.pkgnv = pkgnv
        # find item in pkglist, scroll and select
        it = self.pkglist.get_iter_first()
        while it:
            pkg = (self.pkglist.get_value(it, self.COLUMN_PKG_NAME),
                   self.pkglist.get_value(it, self.COLUMN_PKG_VER))
            if pkg == pkgnv:
                path = self.pkglist.get_path(it)
                self.pkg_tv.scroll_to_cell(path,
                                           self.pkg_tv.get_column(0),
                                           gtk.TRUE,
                                           0.5,
                                           0)
                sel = self.pkg_tv.get_selection()
                sel.select_path(path)
                return
            it = self.pkglist.iter_next(it)
    #! PKGLIST AND COMPSLIST TOOLS

    # PKG AND COMPS FULL DEPS RESOLVING TOOLS
    # here we build self.dep_grpgrp
    def get_grpgrp_deps(self, grp):
        self.d.info4(("get_grpgrp_deps", "group", grp), DBG_COMMON)
        if not self.dep_grpgrp.has_key(grp):
            done = {}
            todo = [grp]
            while todo:
                newones = []
                for it in todo:
                    self.d.info42(2, ("group", it), DBG_LVL1)
                    if not done.has_key(it):
                        done[it] = self.DUMMY
                        if self.dep_grpgrp.has_key(it):
                            for it2 in self.dep_grpgrp[it]:
                                done[it2] = self.DUMMY
                        else:
                            self.d.info42(3, ("groups", self.compsg[it]),
                                          DBG_LVL1)
                            for grp in self.compsg[it]:
                                if self.compsg.has_key(grp):
                                    newones.append(grp)
                                else:
                                    # bad comps, kick this group
                                    self.d.info3("Unknown group: %s (skipped)",
                                                 grp, DBG_ERROR)
                todo = newones
            self.dep_grpgrp[grp] = done.keys()
        self.d.info42(1, ("all deps", self.dep_grpgrp[grp]), DBG_LVL1)
        return self.dep_grpgrp[grp]        

    # here we build self.dep_grppkg
    # when selecting a group:
    #   1- find its group-deps
    #   2- find these groups pkg-deps
    #   3- find these pkgs pkg-deps
    # /!\ step 3 depends on your hdlist !!
    # no pkg version stored in comps.xml so we get it from self.pkgl
    # We only use comps.xml to get the relation between groups
    # and packages.
    def get_grppkg_deps(self, grp):
        self.d.info4(("get_grppkg_deps", "group", grp), DBG_COMMON)
        if not self.dep_grppkg.has_key(grp):
            # 1-
            grpdeps = self.get_grpgrp_deps(grp)
            # 2-
            done = {}
            for it in grpdeps:
                for it2 in self.compsl[it]:
                    done[it2] = self.DUMMY
            # 3-
            done_plus_ver = {}
            for it in self.pkgl.keys():
                if done.has_key(it[0]):
                    done_plus_ver[it] = self.DUMMY
                    for it2 in self.get_pkgpkg_deps(it):
                        done_plus_ver[it2] = self.DUMMY
            self.dep_grppkg[grp] = done_plus_ver.keys()
        return self.dep_grppkg[grp]

    # here we build self.dep_pkgpkg
    def get_pkgpkg_deps(self, pkg):
        self.d.info4(("get_pkgpkg_deps", "pkg", pkg), DBG_COMMON)
        if not self.dep_pkgpkg.has_key(pkg):
            todo = [pkg]
            done = {}
            while todo:
                newones = {}
                for it in todo:
                    self.d.info42(2, ("pkg", it), DBG_LVL1)
                    if not done.has_key(it):
                        if self.dep_pkgpkg.has_key(pkg):
                            self.d.info42(3, ("db", self.dep_pkgpkg[it]),
                                          DBG_LVL1)
                            for it2 in self.dep_pkgpkg[pkg]:
                                done[it2] = self.DUMMY
                        else:
                            childs = self.get_pkg_dep(it)
                            self.d.info42(3, ("", childs), DBG_LVL1)
                            for it2 in childs:
                                newones[it2] = self.DUMMY
                        done[it] = self.DUMMY
                todo = newones.keys()
            self.dep_pkgpkg[pkg] = done.keys()
        return self.dep_pkgpkg[pkg]
    #! PKG AND COMPS FULL DEPS RESOLVING TOOLS
        
    # NOTEBOOK NFO
    def update_glob_info(self, pkg_name):
        pkg = self.pkgl[pkg_name]
        txt = "PACKAGE: %s  [v. %s r. %s]\n" % (pkg.name, pkg.version,
                                                pkg.release)
        txt += "\nGROUP: " + pkg.group + "\n"
        if self.compsp:
            txt += "\nCOMPS.XML GROUP(S): "
            txt += self.format_list(self.compsp[pkg_name]) + "\n"
        txt += "\nSUMMARY: " + pkg.summary + "\n"
        txt += "\nDESCRIPTION:\n" + pkg.description + "\n"
        txt += "\nSIZE:\n"
        if (pkg.fsize / 1024) < 1024:
            txt += "\tRPM: %d KB\n" % (pkg.fsize / 1024, )
            txt += "\tINSTALLED: %d KB\n" % (pkg.size / 1024, )
        else:
            txt += "\tRPM: %d MB\n" % (pkg.fsize / 1024 / 1024, )
            txt += "\tINSTALLED: %d MB\n" % (pkg.size / 1024 / 1024, )
        txt += "\nPROVIDES:\n"
        for it in pkg.provides:
            txt += "\t%s\n" % (it, )
        txt += "\nREQUIRES:\n"
        for it in pkg.requires:
            txt += "\t%s\n" % (it, )
        txt += "\nCONFLICTS:\n"
        for it in pkg.conflicts:
            txt += "\t%s\n" % (it, )
        txt += "\nOBSOLETES:\n"
        for it in pkg.obsoletes:
            txt += "\t%s\n" % (it, )
        self.buff_glob.set_text(txt)
        
    # before calling update*nodes we must make sure all
    # required deps have been resolved
    def update_tree_dep_info(self, val = ""):
        for it in self.page_dep.get_children():
            self.page_dep.remove(it)
        if val:
            self.build_full_dep(val)
            self.update_dep_nodes(val)
            # create ScrolledWindow
            sw = gtk.ScrolledWindow()
            sw.set_border_width(0)
            sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            # create CTree
            ctree = PkgCTree(self.dep_nodes, self.dep_nodes_u, self.expand)
            ctree.set_column_width(0, 380)
            ctree.connect("button_press_event", self.e_tree_click)
            # pack
            sw.add(ctree)
            self.page_dep.add(sw)
            self.page_dep.show_all()

    # before calling update*nodes we must make sure all
    # required deps have been resolved
    def update_tree_rdep_info(self, val = ""):
        for it in self.page_rdep.get_children():
            self.page_rdep.remove(it)
        if val:
            self.build_full_rdep(val)
            self.update_rdep_nodes(val)
            # create ScrolledWindow
            sw = gtk.ScrolledWindow()
            sw.set_border_width(0)
            sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            # create CTree
            ctree = PkgCTree(self.rdep_nodes, self.rdep_nodes_u, self.expand)
            ctree.set_column_width(0, 380)
            ctree.connect("button_press_event", self.e_tree_click)
            # pack
            sw.add(ctree)
            self.page_rdep.add(sw)
            self.page_rdep.show_all()

    def update_group_info(self, group):
	pkgs_m = self.compslm[group]
	pkgs_d = self.compsld[group]
	pkgs_o = self.compslo[group]
	visible = self.user_visible[group]

	# XXX: group display is not coherent with packages.
	#      should be fixed when having separate group dep/rdep windows
        grps = self.compsg[group]
        allgrps = self.get_grpgrp_deps(group)

	pkgs_m.sort(self.pref_sort)
	pkgs_d.sort(self.pref_sort)
	pkgs_o.sort(self.pref_sort)
        grps.sort(self.pref_sort)
        allgrps.sort(self.pref_sort)
        txt = "GROUP: %s (%s)\n" % (group, self.grpname[group])
	txt += "\nUSER VISIBLE: "
	if visible:
	    txt += "yes\n\n"
	else:
	    txt += "no\n\n"
        if pkgs_m:
	    txt += "MANDATORY PACKAGES:\n\n"
            txt += self.format_list(pkgs_m) + "\n\n"
        if pkgs_d:
	    txt += "DEFAULT PACKAGES:\n\n"
            txt += self.format_list(pkgs_d) + "\n\n"
	if pkgs_o:
	    txt += "OPTIONAL PACKAGES:\n\n"
	    txt += self.format_list(pkgs_o) + "\n\n"
        if grps:
	    txt += "GROUPS:\n\n"
            txt += self.format_list(grps) + "\n\n"
        if allgrps:
	    txt += "ALL GROUPS:\n\n"
            txt += self.format_list(allgrps) + "\n\n"
        self.buff_group.set_text(txt)
    #! NOTEBOOK NFO

    # NOTEBOOK REFRESH
    def refresh_notebook_glob(self):
        if self.show_glob and self.pkgnv:
            self.update_glob_info(self.pkgnv)
        else:
            self.set_empty(self.buff_glob)

    def refresh_notebook_tree_dep(self):
        if self.show_dep and self.pkgnv:
            self.update_tree_dep_info(self.pkgnv)
        else:
            self.update_tree_dep_info()

    def refresh_notebook_tree_rdep(self):
        if self.show_rdep and self.pkgnv:
            self.update_tree_rdep_info(self.pkgnv)
        else:
            self.update_tree_rdep_info()

    def refresh_notebook_group(self):
        if self.compsl and self.show_group and self.group:
            self.update_group_info(self.group)
        else:
            self.set_empty(self.buff_group)
            
    def refresh_notebook(self, force = 0):
        if force:
            self.dep_nodes = {}
            self.rdep_nodes = {}
            self.dep_nodes_u = {}
            self.rdep_nodes_u = {}
        self.refresh_notebook_glob()
        self.refresh_notebook_group()
        self.refresh_notebook_tree_dep()
        self.refresh_notebook_tree_rdep()
    #! NOTEBOOK REFRESH

    # MENUBAR TOOLS
    def menubar_item_activate(self, path):
        it = self.item_factory.get_item(path)
        if it:
            it.activate()

    def menubar_item_set_state(self, path, state):
        it = self.item_factory.get_item(path)
        if it:
            it.set_active(state)

    def menubar_item_hide(self, path):
        it = self.item_factory.get_item(path)
        if it:
            it.hide()
            
    def menubar_item_show(self, path):
        it = self.item_factory.get_item(path)
        if it:
            it.show()

    def menubar_item_set_sensitive(self, path, status):
        self.d.info4(("Menubar sensitive status change",
                      path, status), DBG_COMMON)
        it = self.item_factory.get_item(path)
        if it:
            it.set_sensitive(status)
    #! MENUBAR TOOLS

    # STATUSBAR STUFF
    # return filename from path+filename
    def get_file_name(self, fname):
        if fname:
            fn = fname
        else:
            fn = "none"
        pos = string.rfind(fn, "/")
        if pos != -1:
            pos += 1
            fn = fn[pos:]
        return fn

    def refresh_statusbar(self):
        self.statusbar.pop(0)
        if self.rpm_inc:
            rpmdb = "yes"
        else:
            rpmdb = "no"
        data = "HDL: %s -/- CMP: %s -/- DB: %s -/- PKG: %s/%s/%s -/- SIZE: %s/%sMB" % \
        (self.get_file_name(self.file_hdlist),
         self.get_file_name(self.file_comps), rpmdb, self.nb_sel, self.nb_all,
         len(self.pkgl), self.size_sel / (1024 * 1024),
         self.size_all / (1024 *1024))
        self.statusbar.push(0, data)
    #! STATUSBAR STUFF

    # HISTORY
    def get_history_previous(self):
        if self.history_idx <= 0:
            return None
        self.history_idx -= 1
        if self.history_idx == 0:
            self.menubar_item_set_sensitive("/Selection/Previous", gtk.FALSE)
        if self.history_idx == (len(self.history) - 2):
            self.menubar_item_set_sensitive("/Selection/Next", gtk.TRUE)
        return self.history[self.history_idx]
    
    def get_history_next(self):
        if self.history_idx == (len(self.history) - 1):
            return None
        self.history_idx += 1
        if self.history_idx == (len(self.history) - 1):
            self.menubar_item_set_sensitive("/Selection/Next", gtk.FALSE)
        if self.history_idx == 1:
            it = self.menubar_item_set_sensitive("/Selection/Previous",
                                                 gtk.TRUE)
        return self.history[self.history_idx]
            
    def update_history(self):
        self.d.info4(("update history", "history", self.history, "idx",
                      self.history_idx), DBG_COMMON)
        if self.history_idx > -1 and self.history[self.history_idx] == self.pkgnv:
            return
        if self.history_idx == 0:
            self.menubar_item_set_sensitive("/Selection/Previous", gtk.TRUE)
        self.menubar_item_set_sensitive("/Selection/Next", gtk.FALSE)
        if len(self.history):
            self.history = self.history[:self.history_idx + 1]
        self.history += (self.pkgnv, )
        if len(self.history) >= self.history_len:
            self.history = self.history[-self.history_len:]
            self.history_idx = self.history_len -1
        else:
            self.history_idx += 1
        self.d.info42(1, ("new history", self.history, "new idx",
                          self.history_idx), DBG_LVL1)
    #! HISTORY
    
    # EVENTS
    def e_search(self, dummy, widget):
        self.winsearch.show_all()
        
    def e_test(self, dummy, widget):
        # pass
        self.winhtml.show_all()

    def e_show_csv(self, dummy, widget):
        self.wincsv.show_all()

    def e_show_html(self, dummy, widget):
        self.winhtml.show_all()
        
    def e_show_pref(self, dummy, widget):
        self.winpref.show_all()

    def e_collapse_all(self, dummy, widget):
        self.expand_tree(0)

    def e_expand_all(self, dummy, widget):
        self.expand_tree(1)

    def e_sel_all_grp(self, dummy, widget):
        for it in self.comps_sel.keys():
            self.comps_sel[it] = 1
        self.rebuild_comps_grpinc()
        self.rebuild_comps_pkginc()
        self.rebuild_pkg_inc()
        self.refresh_all(force=1)

    def e_sel_all_pkg(self, dummy, widget):
        for it in self.pkg_sel.keys():
            self.pkg_sel[it] = 1
        self.rebuild_pkg_inc()
        self.refresh_all(force=1)
    
    def e_sel_all_deps(self, dummy, widget):
        for it in self.pkg_sel.keys():
            if not self.pkg_sel[it]:
                sel_pkg = (self.pkg_inc or 0) and self.pkg_inc[it]
                sel_comp = (self.comps_pkginc or 0) and self.comps_pkginc[it]
                sel_db = (self.rpm_inc or 0) and self.rpm_inc[it]
                self.pkg_sel[it] = sel_pkg or sel_comp or sel_db
        self.rebuild_pkg_inc()
        self.refresh_all()
    
    def e_sel_prev(self, dummy, widget):
        prev = self.get_history_previous()
        self.d.info4(("e_sel_prev", "history", self.history, "idx",
                      self.history_idx, "prev", prev), DBG_COMMON)
        if prev:
            self.set_pkglist_sel(prev)
            self.refresh_all()
        
    def e_sel_next(self, dummy, widget):
        next = self.get_history_next()
        self.d.info4(("e_sel_next", "history", self.history, "idx",
                      self.history_idx, "next", next), DBG_COMMON)
        if next:
            self.set_pkglist_sel(next)
            self.refresh_all()
            
    def e_unselect_all(self, dummy, widget):
        self.e_unselect_comp(dummy, widget)
        self.e_unselect_pkg(dummy, widget)
        
    def e_unselect_comp(self, dummy, widget):
        if not self.file_comps:
            return
        self.comps_pkginc = {}
        for it in self.pkgl.keys():
            self.comps_pkginc[it] = 0
        self.comps_sel = {}
        self.comps_grpinc = {}
        for it in self.compsl.keys():
            self.comps_grpinc[it] = 0
            self.comps_sel[it] = 0
        self.refresh_all(notebook = 1)

    def e_unselect_pkg(self, dummy, widget):
        if not self.file_hdlist:
            return
        self.pkg_sel = {}
        self.pkg_inc = {}
        for it in self.pkgl.keys():
            self.pkg_inc[it] = 0
            self.pkg_sel[it] = 0
        self.refresh_all(notebook = 1)

    def e_about(self, dummy, widget):
        self.d.info4(("event about", ), DBG_COMMON)
        dialog = gtk.MessageDialog(self.window, gtk.DIALOG_DESTROY_WITH_PARENT,
                                   gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE,
                                   'RPM Analyzer   v.%s\n\nalain.tauch@thalesatm.com' % (self.VERSION, ))
        dialog.connect('response', lambda dialog, response: dialog.destroy())
        dialog.show()

    def e_save_pkg_and_grp_sel(self, dummy, widget):
        self.d.info4(("event export pkg and grp sel", "", self, "", widget),
                     DBG_COMMON)
        self.open_file("Save pkg and grp sel", "", self.save_pkg_and_grp_sel,
                       "", "*.save")

    def e_open_pkg_and_grp_sel(self, dummy, widget):
        self.d.info4(("event open pkg and grp sel", "", self, "", widget),
                     DBG_COMMON)
        self.open_file("Open pkg and grp sel", "", self.open_pkg_and_grp_sel,
                       "", "*.save")

    def e_export_all_sel_ver(self, dummy, widget):
        self.d.info4(("event export all sel - version", "", self, "", widget),
                     DBG_COMMON)
        self.open_file("Save all selected packages as", "",
                       self.export_all_sel, "1")

    def e_export_all_sel_no_ver(self, dummy, widget):
        self.d.info4(("event export all sel - no version", "", self, "", widget),
                     DBG_COMMON)
        self.open_file("Save all selected packages as", "",
                       self.export_all_sel, "")

    # button 1 -> left / 2 -> middle / 3 -> right
    # type BUTTON_PRESS -> 1 / _2BUTTON_PRESS -> / _3BUTTON_PRESS -> 3
    def e_export_tree_dep(self, dummy, widget):
        self.d.info4(("event save tree dep", "", self, "", widget), DBG_COMMON)
        if not self.pkgnv:
            self.send_message("no package selected")
        else:
            self.open_file("Save deps as", "", self.export_tree_dep, "")

    def e_export_tree_rdep(self, dummy, widget):
        self.d.info4(("event save tree rdep", "", self, "", widget), DBG_COMMON)
        if not self.pkgnv:
            self.send_message("no package selected")
        else:
            self.open_file("Save rev deps as", "", self.export_tree_rdep, "")
    
    def e_tree_click(self, widget, event):
        self.d.info4(("event tree click", "", widget), DBG_COMMON)
        if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
            row = widget.focus_row
            node = widget.node_nth(row)
            name = widget.node_get_pixtext(node, 0)[0]
            ver = widget.node_get_text(node, 1)
            self.d.info42(1, ("pkg", name, "version", ver), DBG_LVL1)
            # insert your stuff here
            self.set_pkglist_sel((name, ver))
            self.refresh_all()

    def e_open_local_hdlist(self, dummy = None, widget = None):
        self.d.info4(("event open local hdlist", self), DBG_COMMON)
        self.import_hdlist(self.LOCAL_HDL)
        
    def e_open_local_comps(self, dummy = None, widget = None):
        self.d.info4(("event open local comps.xml", self), DBG_COMMON)
        self.import_comps(self.LOCAL_CMP)
        
    def e_open_local_both(self, dummy, widget):
        self.e_open_local_hdlist()
        self.e_open_local_comps()

    def e_open_hdlist(self, dummy, widget):
        self.d.info4(("event open hdlist", "", self, "", widget), DBG_COMMON)
        self.open_file("Open hdlist", "", self.import_hdlist, "")

    def e_open_comps(self, dummy, widget):
        self.d.info4(("event open comps", "", self, "", widget), DBG_COMMON)
        self.open_file("Open comps.xml", "", self.import_comps, "", "*.xml")

    def e_open_db(self, dummy, widget):
        self.d.info4(("event open local db", "", self, "", widget), DBG_COMMON)
        if self.rpm_inc:
            self.rpm_inc = {}
        else:
            self.import_db()
        self.refresh_all(notebook = 1)

    def e_menubar_display_toggle_glob(self, dummy, widget):
        self.d.info4(("event menubar_display_toggle_glob", "", id, "", widget),
                     DBG_COMMON)
        self.show_glob = widget.get_active()
        if self.show_glob:
            self.page_glob.show()
        else:
            self.page_glob.hide()
        self.refresh_notebook_glob()

    def e_menubar_display_toggle_dep(self, dummy, widget):
        self.d.info4(("event menubar_display_toggle_dep", "", id, "", widget),
                     DBG_COMMON)
        self.show_dep = widget.get_active()
        if self.show_dep:
            self.page_dep.show()
        else:
            self.page_dep.hide()
        self.refresh_notebook_tree_dep()

    def e_menubar_display_toggle_rdep(self, dummy, widget):
        self.d.info4(("event menubar_display_toggle_rdep", "", id, "", widget),
                     DBG_COMMON)
        self.show_rdep = widget.get_active()
        if self.show_rdep:
            self.page_rdep.show()
        else:
            self.page_rdep.hide()
        self.refresh_notebook_tree_rdep()

    def e_menubar_display_toggle_group(self, dummy, widget):
        self.d.info4(("event menubar_display_toggle_group", "", id, "", widget),
                     DBG_COMMON)
        self.show_group = widget.get_active()
        if self.show_group:
            self.page_group.show()
        else:
            self.page_group.hide()
        self.refresh_notebook_group()
            
    def e_list_sel_change(self, selection):
        self.d.info4(("event list sel change", "", self, "", selection),
                     DBG_COMMON)
        (model, it) = selection.get_selected()
        if it:
            self.pkgnv = (model.get_value(it, self.COLUMN_PKG_NAME),
                          model.get_value(it, self.COLUMN_PKG_VER))
            self.d.info3("\t\\_ %s\n\t\\_ %s [v. %s]", (it,) + self.pkgnv,
                         DBG_LVL1)
        else:
            self.d.info42(1, ("", "no selection"), DBG_LVL1)
            self.pkgnv = ()
        # notebook focus
        if self.notebook.get_current_page() == self.NOTEBOOK_PAGE_GROUP:
            if self.show_glob:
                self.notebook.set_current_page(self.NOTEBOOK_PAGE_GLOB)
            elif self.show_dep:
                self.notebook.set_current_page(self.NOTEBOOK_PAGE_DEP)
            elif self.show_dep:
                self.notebook.set_current_page(self.NOTEBOOK_PAGE_RDEP)
        self.refresh_notebook()
        self.update_history()
        self.d.print_refcounts(60, DBG_LVL5)

    def e_list_toggle(self, cell, path, model):
        self.d.info4(("e_list_toggle", "cell", cell, "path", path, "model",
                      model), DBG_COMMON)
        iter = model.get_iter((int(path), ))
        clicked = model.get_value(iter, self.COLUMN_PKG_SEL)
        pkg = (model.get_value(iter, self.COLUMN_PKG_NAME),
               model.get_value(iter, self.COLUMN_PKG_VER))
        clicked = not clicked
        # set the new value in sel list
        self.pkg_sel[pkg] = clicked
        # build its dep tree
        self.build_full_dep(pkg)
        # update self.pkg_inc
        self.rebuild_pkg_inc()
        # update pkglist because we may have changed the whole thing
        # don't update the whole list, just the selection cols
        self.refresh_pkglist()
        # update notebook, same here
        self.refresh_notebook(force = 1)
        self.refresh_statusbar()
        # do not select when toggle item
        self.toggle = (int(path), )

    def e_comps_sel_change(self, selection):
        self.d.info4(("event comps sel change", "", self, "", selection),
                     DBG_COMMON)
        (model, it) = selection.get_selected()
        if it:
            self.group = model.get_value(it, self.COLUMN_PKG_NAME)
            self.d.info42(1, ("", it, "", self.group), DBG_LVL1)
        else:
            self.d.info42(1, ("", "no selection"), DBG_LVL1)
            self.group = ()
        # notebook focus
        if self.show_group:
            self.notebook.set_current_page(self.NOTEBOOK_PAGE_GROUP)
        self.refresh_notebook_group()
        
    def e_comps_toggle(self, cell, path, model):
        self.d.info4(("event comps_toggle", "", self, "cell", cell, "path",
                      path, "model", model), DBG_COMMON)
        iter = model.get_iter((int(path), ))
        clicked = model.get_value(iter, self.COLUMN_COMPS_SEL)
        grp = model.get_value(iter, self.COLUMN_COMPS_NAME)
        clicked = not clicked
        self.comps_sel[grp] = clicked
        self.rebuild_comps_grpinc()
        self.rebuild_comps_pkginc()
        # redraw the sel cols only + notebook
        self.refresh_all(notebook = 1)

    def e_destroy(self, widget):
        self.d.info4(("event destroy", "", self, "", widget), DBG_COMMON)
        self.window.hide()
        gtk.main_quit()

    # Window Manager behaviour
    def e_delete(self, widget, event):
        self.d.info4(("event delete", "", self, "", widget, "", event),
                     DBG_COMMON)
        self.e_destroy(widget)
        return gtk.TRUE

    # Window Manager behaviour
    def e_hide(self, widget, event):
        self.d.info4(("event hide", "", self, "", widget, "", event),
                     DBG_COMMON)
        widget.hide()
        return gtk.TRUE
    #! EVENTS

    # INTERFACE
    def init_iface_part_tlt(self):
        # pkglist comes here
        self.pkglist = gtk.ListStore(gobject.TYPE_BOOLEAN,
                                     gobject.TYPE_BOOLEAN,
                                     gobject.TYPE_STRING,
                                     gobject.TYPE_STRING,
                                     gobject.TYPE_STRING,
                                     gobject.TYPE_STRING,
                                     gobject.TYPE_STRING)
        tv = gtk.TreeView(self.pkglist)
        # altern row colors
        tv.set_rules_hint(gtk.TRUE)
        # COLUMN_PKG_SEL
        renderer = gtk.CellRendererToggle ()
        renderer.connect('toggled', self.e_list_toggle, self.pkglist)
        column0 = gtk.TreeViewColumn("Sel", renderer,
                                     active = self.COLUMN_PKG_SEL)
        column0.set_clickable(gtk.TRUE)
        column0.set_fixed_width(50)
        tv.insert_column(column0, self.COLUMN_PKG_SEL)
        # COLUMN_PKG_ALL
        #renderer = gtk.CellRendererToggle ()
        #renderer.connect('toggled', self.e_list_toggle, self.pkglist)
        renderer = gtk.CellRendererToggle ()
        column3 = gtk.TreeViewColumn("Dep", renderer,
                                     active = self.COLUMN_PKG_ALL)
        column3.set_clickable(gtk.FALSE)
        column3.set_fixed_width(50)
        tv.insert_column(column3, self.COLUMN_PKG_ALL)
        # COLUMN_PKG_NAME
        cell = gtk.CellRendererText()
        column1 = gtk.TreeViewColumn("RPM", cell, text = self.COLUMN_PKG_NAME)
        column1.set_resizable(gtk.TRUE)
        tv.insert_column(column1, self.COLUMN_PKG_NAME)
        # COLUMN_PKG_VER
        column2 = gtk.TreeViewColumn("version", cell,
                                     text = self.COLUMN_PKG_VER)
        column2.set_resizable(gtk.TRUE)
        tv.insert_column(column2, self.COLUMN_PKG_VER)
        # COLUMN_PKG_RELEASE 
        column4 = gtk.TreeViewColumn("release", cell,
                                     text = self.COLUMN_PKG_RELEASE)
        column4.set_resizable(gtk.TRUE)
        tv.insert_column(column4, self.COLUMN_PKG_RELEASE)
        # COLUMN_PKG_FSIZE
        column5 = gtk.TreeViewColumn("File Size", cell,
                                     text = self.COLUMN_PKG_FSIZE)
        column5.set_resizable(gtk.TRUE)
        tv.insert_column(column5, self.COLUMN_PKG_FSIZE)
        # COLUMN_PKG_ISIZE
        column6 = gtk.TreeViewColumn("Installed Size", cell,
                                     text = self.COLUMN_PKG_ISIZE)
        column6.set_resizable(gtk.TRUE)
        tv.insert_column(column6, self.COLUMN_PKG_ISIZE)
        # don t select when toggle
        sel = tv.get_selection()
        sel.set_select_function(self.toggle_check, None)
        sel.connect("changed", self.e_list_sel_change)
        # keep track
        self.pkg_tv = tv
        # scrolled window 1
        sw = gtk.ScrolledWindow()
        sw.set_border_width(0)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(tv)
        sw.set_size_request(310, 350)
        return sw

    def init_iface_part_tlb(self):
        # add the comps frame
        self.compslist = gtk.ListStore(gobject.TYPE_BOOLEAN,
                                       gobject.TYPE_BOOLEAN,
                                       gobject.TYPE_STRING)
        tv = gtk.TreeView(self.compslist)
        # altern row colors
        tv.set_rules_hint(gtk.TRUE)
        # COLUMN_COMPS_SEL
        renderer = gtk.CellRendererToggle ()
        renderer.connect('toggled', self.e_comps_toggle, self.compslist)
        column0 = gtk.TreeViewColumn("Sel", renderer,
                                     active = self.COLUMN_COMPS_SEL)
        column0.set_clickable(gtk.TRUE)
        column0.set_fixed_width(50)
        tv.insert_column(column0, self.COLUMN_COMPS_SEL)
        # COLUMN_COMPS_SEL
        renderer = gtk.CellRendererToggle ()
        column1 = gtk.TreeViewColumn("Dep", renderer,
                                     active = self.COLUMN_COMPS_ALL)
        column1.set_clickable(gtk.FALSE)
        column1.set_fixed_width(50)
        tv.insert_column(column1, self.COLUMN_COMPS_ALL)
        # COLUMN_COMPS_NAME
        cell = gtk.CellRendererText()
        column2 = gtk.TreeViewColumn("Group", cell,
                                     text = self.COLUMN_COMPS_NAME)
        tv.insert_column(column2, self.COLUMN_COMPS_NAME)
        # selection
        sel = tv.get_selection()
        sel.connect("changed", self.e_comps_sel_change)
        # scrolled window 
        sw = gtk.ScrolledWindow()
        sw.set_border_width(0)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(tv)
        sw.set_size_request(310, 150)
        return sw

    def init_iface_part_tl(self):
        vpaned = gtk.VPaned()
        tlt = self.init_iface_part_tlt()
        tlb = self.init_iface_part_tlb()
        vpaned.add(tlb)
        vpaned.add(tlt)
        return vpaned

    def init_iface_part_tr(self):
        # notebook comes here
        self.notebook = gtk.Notebook()
        self.notebook.set_size_request(650, -1)
        # notebook global info page
        self.page_glob, self.buff_glob = self.create_text(gtk.FALSE)
        l = gtk.Label('')
        l.set_text_with_mnemonic('_General')
        self.notebook.insert_page(self.page_glob, l, self.NOTEBOOK_PAGE_GLOB)
        # notebook tree dep page
        eb = gtk.EventBox()
        self.tooltips.set_tip(eb, self.TOOLTIP_DEP)        
        l = gtk.Label('')
        l.set_text_with_mnemonic('Tree _Dep')
        self.page_dep = eb
        self.notebook.insert_page(self.page_dep, l, self.NOTEBOOK_PAGE_DEP)
        # notebook tree rdep page 
        eb2 = gtk.EventBox()
        self.tooltips.set_tip(eb2, self.TOOLTIP_DEP)        
        l = gtk.Label('')
        l.set_text_with_mnemonic('Tree _RDep')
        self.page_rdep = eb2
        self.notebook.insert_page(self.page_rdep, l, self.NOTEBOOK_PAGE_RDEP)
        # notebook group info page
        self.page_group, self.buff_group = self.create_text(gtk.FALSE)
        l = gtk.Label('')
        l.set_text_with_mnemonic('_Group')
        self.notebook.insert_page(self.page_group, l, self.NOTEBOOK_PAGE_GROUP)
        return self.notebook

    def init_iface_part_t(self):
        hpaned = gtk.HPaned()
        tl = self.init_iface_part_tl()
        tr = self.init_iface_part_tr()        
        hpaned.add(tl)
        hpaned.add(tr)
        return hpaned

    def init_iface_part_b(self):
        self.statusbar = gtk.Statusbar()
        # event box needed for tooltips
        eb = gtk.EventBox()
        eb.add(self.statusbar)
        self.tooltips.set_tip(eb, self.TOOLTIP_STATUS)
        return eb

    # menubar
    def init_iface_part_m(self):
        self.menu_items = (
            ("/_File", None, None, 0, "<Branch>"),
            ("/File/Open _Hdlist", "<control>H", self.e_open_hdlist, 0, None),
            ("/File/Open _Comps.xml", "<control>C", self.e_open_comps, 0, None),
            ("/File/sep1", None, None, 0, "<Separator>"),
            ("/File/Open local/Hdlist", None, self.e_open_local_hdlist, 0, None),
            ("/File/Open local/Comps.xml", None, self.e_open_local_comps, 0, None),
            ("/File/Open local/sep1", None, None, 0, "<Separator>"),
            ("/File/Open local/_Both", "<control>B", self.e_open_local_both, 0, None),
            ("/File/Toggle local _Db on\/off", "<control>D", self.e_open_db, 0, None),
            ("/File/sep2", None, None, 0, "<Separator>"),
            ("/File/_Save current selection", "<control>S", self.e_save_pkg_and_grp_sel, 0, None),
            ("/File/_Open selection", "<control>O", self.e_open_pkg_and_grp_sel, 0, None),
            ("/File/sep3", None, None, 0, "<Separator>"),
            ("/File/Quit", "<control>Q", self.quit, 0, None),
            ("/De_pth", None, None, 0, "<Branch>"),
            ("/Depth/Full", None, self.set_tree_depth_all, 0, "<RadioItem>"),
            ("/Depth/1 level", None, self.set_tree_depth_one, 0, "/Depth/Full"),
            ("/Depth/2 levels", None, self.set_tree_depth_two, 0, "/Depth/Full"),
            ("/Depth/3 levels", None, self.set_tree_depth_three, 0, "/Depth/Full"),
            ("/_Display", None, None, 0, "<Branch>"),
            ("/Display/General", None, self.e_menubar_display_toggle_glob, 0, "<CheckItem>"),
            ("/Display/Deps", None, self.e_menubar_display_toggle_dep, 0, "<CheckItem>"),
            ("/Display/Rdeps", None, self.e_menubar_display_toggle_rdep, 0, "<CheckItem>"),
            ("/Display/Group", None, self.e_menubar_display_toggle_group, 0, "<CheckItem>"),
            ("/Display/sep1", None, None, 0, "<Separator>"),
            ("/Display/Collapse all", None, self.e_collapse_all, 0, None),
            ("/Display/Expand all", None, self.e_expand_all, 0, None),
            ("/Display/sep2", None, None, 0, "<Separator>"),
            ("/Display/_Preferences", None, self.e_show_pref, 0, None),
# test only
#            ("/Display/sep3", None, None, 0, "<Separator>"),
#            ("/Display/_Test", "<control>T", self.e_test, 0, None),
#! test only
            ("/_Selection", None, None, 0, "<Branch>"),
            ("/Selection/_Previous", "<control>P", self.e_sel_prev, 0, None),
            ("/Selection/_Next", "<control>N", self.e_sel_next, 0, None),
            ("/Selection/_Find", "<control>F", self.e_search, 0, None),
            ("/Selection/sep1", None, None, 0, "<Separator>"),
            ("/Selection/Unselect _all", None, self.e_unselect_all, 0, None),
            ("/Selection/Unselect _packages", None, self.e_unselect_pkg, 0, None),
            ("/Selection/Unselect _groups", None, self.e_unselect_comp, 0, None),
            ("/Selection/sep2", None, None, 0, "<Separator>"),
            ("/Selection/Select all _deps", None, self.e_sel_all_deps, 0, None),
            ("/Selection/Select all _packages", None, self.e_sel_all_pkg, 0, None),
            ("/Selection/Select all _groups", None, self.e_sel_all_grp, 0, None),
            ("/_Export", None, None, 0, "<Branch>"),
            ("/Export/_dependency tree (ascii)", None, self.e_export_tree_dep, 0, None),
            ("/Export/_rev dependency tree (ascii)", None, self.e_export_tree_rdep, 0, None),
            ("/Export/sep1", None, None, 0, "<Separator>"),
            ("/Export/all selected packages", None, self.e_export_all_sel_no_ver, 0, None),
            ("/Export/all selected packages (+ version)", None, self.e_export_all_sel_ver, 0, None),
            ("/Export/selection status (html)", None, self.e_show_html, 0, None),
            ("/Export/selection status (csv)", None, self.e_show_csv, 0, None),
            ("/_Help", None, None, 0, "<LastBranch>"),
            ("/Help/_About", None, self.e_about, 0, None),
            )
        accel_group = gtk.AccelGroup()
        item_factory = gtk.ItemFactory(gtk.MenuBar, "<main>", accel_group)
        item_factory.create_items(self.menu_items)
        self.window.add_accel_group(accel_group)
        self.item_factory = item_factory
        return item_factory.get_widget("<main>")

    def init_menubar_states(self):
        # depth
        if self.depth_tree in [2, 3]:
            path = "/Depth/%s levels" % (self.depth_tree,)
        elif self.depth_tree == 1:
            path = "/Depth/1 level"
        else:
            path = "/Depth/Full"
        self.menubar_item_activate(path)
        # display
        self.menubar_item_set_state("/Display/General", self.show_glob)
        self.menubar_item_set_state("/Display/Deps", self.show_dep)
        self.menubar_item_set_state("/Display/Rdeps", self.show_rdep)
        self.menubar_item_set_state("/Display/Group", self.show_group)
        self.menubar_item_set_sensitive("/Selection/Previous", gtk.FALSE)
        self.menubar_item_set_sensitive("/Selection/Next", gtk.FALSE)
        ok = self.check_file(self.LOCAL_HDL, 'r')
        self.menubar_item_set_sensitive("/File/Open local", ok)
        self.menubar_item_set_sensitive("/File/Open local/Both", ok)
        self.menubar_item_set_sensitive("/Selection", gtk.FALSE)
        self.menubar_item_set_sensitive("/Export", gtk.FALSE)

    def init_winpref_states(self):
        # we already have someone for this
        self.on_pref_change_pkg()

    def init_iface(self):
        #main window
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        # tooltips
        self.tooltips = gtk.Tooltips()
        # set window properties
        self.window.set_border_width(0)
        self.window.set_title("RPM Analyzer")
        self.window.set_default_size(800, 600)
        # connect events
        self.window.connect("delete_event", self.e_delete)
        self.window.connect("destroy", self.e_destroy)   
        # make a table/grid
        table = gtk.Table(3, 4, gtk.FALSE)
        table.set_row_spacing(2, 5)
        table.set_col_spacing(0, 5)
        self.window.add(table)
        # top part
        t = self.init_iface_part_t()
        table.attach(t,
                     # X direction              Y direction
                     0, 3,                      1, 3,
                     gtk.FILL,                  gtk.EXPAND | gtk.FILL,
                     0,                         0)
        # bottom part
        b = self.init_iface_part_b()
        table.attach(b,
                     0, 3,                      3, 4,
                     gtk.EXPAND | gtk.FILL,     0,
                     0,                         0)        
        # menubar
        m = self.init_iface_part_m()
        table.attach(m,
                     # X direction              Y direction
                     0, 3,                      0, 1,
                     gtk.EXPAND | gtk.FILL,     gtk.FILL,
                     0,                         0)
        self.tooltips.enable()
        # don't forget the preferences window
        self.winpref = WinPref(self.on_pref_change)
        # time for the html export window too
        self.winhtml = WinHtml(self.on_export_html)
        # time for the csv export window
        self.wincsv = WinCsv(self.on_export_csv)
        # create winsearch too
        self.winsearch = WinSearch(self.on_search)

        # load saved prefs
        if posixpath.exists(self.pref_file):
#            try:
            self.winpref.restore_from_file(self.pref_file)
#            except:
#                self.d.info42(1, ("ERROR", "Unable to load preferences ",
#                                  self.pref_file), DBG_ERROR)

        # initialise iface according to prefs
        self.init_winpref_states()

    def show_iface(self): 
	self.window.move(0, 0)
        self.window.show_all()
        if not self.show_glob:
            self.page_glob.hide()
        if not self.show_dep:
            self.page_dep.hide()
        if not self.show_rdep:
            self.page_rdep.hide()
        if not self.show_group:
            self.page_group.hide()        
        # set initial menubar states
        self.init_menubar_states()
    #! INTERFACE

    # INIT
    # when adding new members, update clean_members as well
    # to be sure to clean the environnement when a new hdlist
    # is loaded
    def init_members(self):
        self.pkgl = {}
        self.pkg_inc = {}
        self.pkg_sel = {}

        self.expand = 0

        self.dep_tree = {}
        self.rdep_tree = {}
        self.dep_nodes = {}
        self.rdep_nodes = {}
        self.dep_nodes_u = {}
        self.rdep_nodes_u = {}
        self.compsl = {}
        self.compsg = {}
        self.comps_grpinc = {}
        self.comps_pkginc = {}
        self.comps_sel = {}
        self.group = None

        self.dep_pkgpkg = {}
        self.dep_grppkg = {}
        self.dep_grpgrp = {}
        self.show_glob = gtk.TRUE
        self.show_dep = gtk.FALSE
        self.show_rdep = gtk.FALSE
        self.show_group = gtk.FALSE

        self.depth_tree = -1
        self.d = debug.Debug(DEBUG_LEVEL)
        self.pkgnv = ()

        self.file_hdlist = ""
        self.file_comps = ""
        # stats
        self.nb_sel = 0
        self.nb_all = 0
        self.size_sel = 0
        self.size_all = 0
        # local rpm db
        self.rpm_inc = {}
        # flag: set when toggle, checked before selection
        self.toggle = None
        # tooltips
        self.tooltips = None
        # history
        self.history_len = 5
        self.history_idx = -1
        self.history = ()
        # last opened file path
        self.last_path = ""

        # home directory
        self.homedir = posixpath.expanduser("~/.rpm-analyzer/")
        if not posixpath.exists(self.homedir):
            try:
                os.makedirs(self.homedir)
            except:
                self.d.info42(1, ("ERROR", "Unable to create home directory",
                                  "path", self.homedir), DBG_ERROR)
        # pref file
        self.pref_file = self.homedir + "preferences"

    def __init__(self):
        # init here members for best visibility
        self.init_members()
        # iface
        self.init_iface()

        # show all
        self.show_iface()
    #! INIT

    def usage(self):
        print "rpm-analyzer v%s" % self.VERSION
        print "  [-h <hdlist>    | --hdlist=<hdlist>  ] : open hdlist"
        print "  [-c <comps.xml> | --comps=<comps.xml>] : open comps.xml"
        print "or:"
        print "  [-l             | --local]             : open local files"
        print "  [-v             | --version ]          : display version"
        sys.exit(1)
        
    def main(self):
        try:
            opts, args = getopt.getopt(sys.argv[1:],
                                       "h:c:lv",
                                       ["hdlist=", "comps=", "local", "version"])
        except getopt.GetoptError:
            self.usage()
        hdlist = ""
        comps  = ""
        local  = FALSE
        for option, arg in opts:
            if option in ("-h", "--hdlist"):
                hdlist = arg
            if option in ("-c", "--comps"):
                comps = arg
            if option in ("-l", "--local"):
                local = TRUE
            if option in ("-v", "--version"):
                print "rpm-analyzer v%s" % self.VERSION
                sys.exit(1)
        if hdlist:
            if local:
                self.usage()
            self.import_hdlist(hdlist)
        if comps:
            if local:
                self.usage()
            self.import_comps(comps)
        if local:
            self.e_open_local_hdlist()
            self.e_open_local_comps()            
        gtk.main()

# for command line execution
if __name__ == "__main__":
    ra = RPMAnalyzer()
    ra.main()
# profiler
#    profile.run('ra.main()', 'toto')
#    p = pstats.Stats('toto')
#    p.print_stats()

