#
#   File        : winchoice.py
#   Author      : Alain TAUCH <alain.tauch@thalesatm.com>
#   Description : the WinChoice class
#
#   Copyright (C) 2003 Alain TAUCH
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import gtk
import gobject
from gtk import TRUE, FALSE

# two types of choiceboxes: single/multiple
CHOICE_SINGLE       = 0
CHOICE_MULTI        = 1

DEFAULT_TITLE       = "User Input requested"
DUMMY               = 1
    

class WinChoice:

    # type: single/multi
    # descr: description
    # ans: answers [(TRUE/FALSE, answer1), ...]
    # title: window title
    def __init__(self, type, descr, ans, title=None):
        self.type = type
        self.descr = descr
        self.ans = ans
        if title:
            self.title = title
        else:
            self.title = DEFAULT_TITLE
        # store {Check,Radio}Buttons
        self.items = []
    
    def create_window(self):
        """private: create window"""
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        # set window properties
        self.window.set_border_width(8)
        self.window.set_title(self.title)
        # this shouldn't be an option
        self.window.set_modal(TRUE)
        # connect events
        self.window.connect("delete_event", self.e_clicked_ok)
        # content
        vbox_main = gtk.VBox()
        vbox_main.set_spacing(4)
        self.window.add(vbox_main)
        # description
        label = gtk.Label(self.descr)
        label.set_line_wrap(TRUE)
        label.set_justify(gtk.JUSTIFY_FILL)
        vbox_main.add(label)
        # answers
        for it in self.ans:
            if self.type == CHOICE_SINGLE:
                if len(self.items):
                    itm = gtk.RadioButton(self.items[0], it[1])
                else:
                    itm = gtk.RadioButton(None, it[1])
            else:
                itm = gtk.CheckButton(it[1])
            self.items += [itm, ]
            itm.set_active(it[0])
            vbox_main.add(itm)
        # ok button
        ok  = gtk.Button(stock="gtk-ok")
        ok.connect("clicked", self.e_clicked_ok)
        vbox_main.add(ok)
        
    def run(self):
        """main routine, create window and return result"""
        self.create_window()
        self.window.show_all()
        gtk.main()
        res = self.get_status()
        self.window.hide()
        self.window.destroy()
        return res

    # internal use only
    def get_status(self):
        """private: retrieve {radio,check}buttons status"""
        res = ()
        for it in self.items:
            res += (it.get_active(), )
        return res

    # EVENTS
    def e_none(self, widget, event):
        """private: fake event, no more used"""
        pass

    def e_clicked_ok(self, widget, event=None):
        """private: exit from loop"""
        gtk.main_quit()
    #! EVENTS


