#
#   File        : wincsv.py
#   Author      : Alain TAUCH <alain.tauch@thalesatm.com>
#   Description : the WinCsv class
#
#   Copyright (C) 2006 Alain TAUCH
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   * Thu Jun 08 2006 Alain TAUCH <alain.tauch@thalesatm.com>
#   - delete_event mishandle fix
#


import gtk
import gobject
import os

from gtk import TRUE, FALSE

class WinCsv:
    
    DUMMY        = 1

    def __init__(self, callback):
        self.callback = callback
        self.init_members()
        self.create_window()
        self.set_status()

    # set default values
    def init_members(self):
        self.file = ""
        self.deps = TRUE
        self.version = TRUE
        self.release = FALSE
        self.fsize = FALSE
        self.isize = FALSE
        self.description = TRUE
        self.last_path = ""

    # we'd like to change some, depending on pref
    def set_pref(self, version, release, fsize, isize):
        self.version = version
        self.release = release
        self.fsize = fsize
        self.isize = isize

    def create_window(self):
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        # set window properties
        self.window.set_border_width(8)
        self.window.set_title("RPM Analyzer - CSV Export")
#        self.window.set_default_size(400, 200)
        self.window.set_modal(FALSE)
        # connect events
        self.window.connect("delete_event", self.e_hide)
        # content
        # VBox for all components 
        vbox_main = gtk.VBox()
        vbox_main.set_spacing(4)
        self.window.add(vbox_main)
        # csv frame
        frame_csv = gtk.Frame("CSV Export")
        vbox_csv = gtk.VBox()
        vbox_csv.set_spacing(10)
        #   file selection
        hbox_csv = gtk.HBox()
        hbox_csv.set_spacing(10)
        txt = gtk.Label("File:")
        hbox_csv.pack_start(txt)
        self.entry_csv = gtk.Entry()
        hbox_csv.pack_start(self.entry_csv)
        browse = gtk.Button(stock="gtk-open")
        browse.connect("pressed", self.e_browse)
        hbox_csv.pack_start(browse, expand=FALSE, fill=FALSE)
        vbox_csv.pack_start(hbox_csv)
        #   deps inclusion
        self.cb_deps = gtk.CheckButton("include dependencies as selection")
        vbox_csv.pack_start(self.cb_deps)
        #   version
        self.cb_version = gtk.CheckButton("include version")
        vbox_csv.pack_start(self.cb_version)
        #   release
        self.cb_release = gtk.CheckButton("include release")
        vbox_csv.pack_start(self.cb_release)
        #   fsize
        self.cb_fsize = gtk.CheckButton("include file/rpm size")
        vbox_csv.pack_start(self.cb_fsize)
        #   isize
        self.cb_isize = gtk.CheckButton("include installed size")
        vbox_csv.pack_start(self.cb_isize)
        #   description
        self.cb_description = gtk.CheckButton("include description")
        vbox_csv.pack_start(self.cb_description)
        #
        frame_csv.add(vbox_csv)
        vbox_main.add(frame_csv)
        # bottom buttons -> HBox
        hbox_buttons = gtk.HBox()
        hbox_buttons.set_spacing(4)
        ok = gtk.Button(stock="gtk-ok")
        ok.connect("pressed", self.e_pressed_ok)
        hbox_buttons.add(ok)
        cancel = gtk.Button(stock="gtk-cancel")
        cancel.connect("pressed", self.e_pressed_cancel)
        hbox_buttons.add(cancel)
        vbox_main.pack_end(hbox_buttons)
        # warning text

    def show_all(self):
        self.set_status()
        self.window.show_all()
        self.window.get_focus()
        
    def get_status(self):
        self.file = self.entry_csv.get_text()
        self.deps = self.cb_deps.get_active()
        self.version = self.cb_version.get_active()
        self.release = self.cb_release.get_active()
        self.fsize = self.cb_fsize.get_active()
        self.isize = self.cb_isize.get_active()
        self.description = self.cb_description.get_active()

    def set_status(self):
        self.entry_csv.set_text(self.file)
        self.cb_deps.set_active(self.deps)
        self.cb_version.set_active(self.version)
        self.cb_release.set_active(self.release)
        self.cb_fsize.set_active(self.fsize)
        self.cb_isize.set_active(self.isize)
        self.cb_description.set_active(self.description)

    # EVENTS
    # Window Manager behaviour
    def e_browse(self, widget):
        filesel = gtk.FileSelection("Select File")
        filesel.set_filename(self.last_path)
        filesel.complete("*.csv")
        result = filesel.run()
        if result == gtk.RESPONSE_OK:
            self.file = filesel.get_filename()
            self.entry_csv.set_text(self.file)
            self.last_path = os.path.dirname(self.file) + "/" 
        filesel.destroy()
        
    def e_hide(self, widget, event):
        widget.hide()
        # do not destroy this window
        return TRUE

    def e_pressed_ok(self, widget):
        self.get_status()
        self.e_hide(self.window, self.DUMMY)
        self.callback()

    def e_pressed_cancel(self, widget):
        self.e_hide(self.window, self.DUMMY)
    #! EVENTS
