#
#   File        : winhtml.py
#   Author      : Alain TAUCH <alain.tauch@thalesatm.com>
#   Description : the WinHtml class
#
#   Copyright (C) 2003-2006 Alain TAUCH
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   * Thu Jun 08 2006 Alain TAUCH <alain.tauch@thalesatm.com>
#   - delete_event mishandle fix
#


import gtk
import gobject
import os

from gtk import TRUE, FALSE

class WinHtml:
    
    DUMMY        = 1
    FL_NODEPS    = 1 << 0
    FL_DEPSASSEL = 1 << 1
    FL_SHOWDEPS  = 1 << 2

    def __init__(self, callback):
        self.callback = callback
        self.init_members()
        self.create_window()
        self.set_status()

    # set default values
    def init_members(self):
        self.file = ""
        self.css = gtk.FALSE
        self.sel_only = gtk.FALSE
        self.deps = self.FL_NODEPS
        self.last_path = ""
        # add release to version nb
        self.rel = gtk.FALSE

    def create_window(self):
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        # set window properties
        self.window.set_border_width(8)
        self.window.set_title("RPM Analyzer - HTML Export")
#        self.window.set_default_size(400, 200)
        self.window.set_modal(gtk.FALSE)
        # connect events
        self.window.connect("delete_event", self.e_hide)
        # content
        # VBox for all components 
        vbox_main = gtk.VBox()
        vbox_main.set_spacing(4)
        self.window.add(vbox_main)
        # html frame
        frame_html = gtk.Frame("HTML Export")
        vbox_html = gtk.VBox()
        vbox_html.set_spacing(10)
        #   file selection
        hbox_html = gtk.HBox()
        hbox_html.set_spacing(10)
        txt = gtk.Label("File:")
        hbox_html.pack_start(txt)
        self.entry_html = gtk.Entry()
        hbox_html.pack_start(self.entry_html)
        browse = gtk.Button(stock="gtk-open")
        browse.connect("pressed", self.e_browse)
        hbox_html.pack_start(browse, expand=gtk.FALSE, fill=gtk.FALSE)
        vbox_html.pack_start(hbox_html)
        #   css use
        self.cb_css = gtk.CheckButton("use CSS")
        vbox_html.pack_start(self.cb_css)
        #   selected packages only
        self.cb_sel_only = gtk.CheckButton("export selection only")
        vbox_html.pack_start(self.cb_sel_only)
        #   add release to version (v-r)
        self.cb_rel = gtk.CheckButton("add release to version number")
        vbox_html.pack_start(self.cb_rel)
        #   dependencies
        self.rb_nodeps = gtk.RadioButton(None, "do not show dependencies")
        self.rb_deps = gtk.RadioButton(self.rb_nodeps, "show dependencies")
        self.rb_deps_as_sel = gtk.RadioButton(self.rb_nodeps,
                                              "consider dependencies "
                                              "as selection")
        vbox_html.pack_start(self.rb_nodeps)
        vbox_html.pack_start(self.rb_deps)
        vbox_html.pack_start(self.rb_deps_as_sel)
        #
        frame_html.add(vbox_html)
        vbox_main.add(frame_html)
        # bottom buttons -> HBox
        hbox_buttons = gtk.HBox()
        hbox_buttons.set_spacing(4)
        ok = gtk.Button(stock="gtk-ok")
        ok.connect("pressed", self.e_pressed_ok)
        hbox_buttons.add(ok)
        cancel = gtk.Button(stock="gtk-cancel")
        cancel.connect("pressed", self.e_pressed_cancel)
        hbox_buttons.add(cancel)
        vbox_main.pack_end(hbox_buttons)
        # warning text

    def show_all(self):
        self.window.show_all()
        self.window.get_focus()
        
    def get_status(self):
        self.file = self.entry_html.get_text()
        self.css = self.cb_css.get_active()
        self.sel_only = self.cb_sel_only.get_active()
        self.rel = self.cb_rel.get_active()
        if self.rb_nodeps.get_active():
            self.deps = self.FL_NODEPS
        elif self.rb_deps.get_active():
            self.deps = self.FL_SHOWDEPS
        else:
            self.deps = self.FL_DEPSASSEL

    def set_status(self):
        self.entry_html.set_text(self.file)
        self.cb_css.set_active(self.css)
        self.cb_sel_only.set_active(self.sel_only)
        self.cb_rel.set_active(self.rel)
        if self.deps == self.FL_NODEPS:
            self.rb_nodeps.set_active(gtk.TRUE)
        elif self.deps == self.FL_DEPSASSEL:
            self.rb_deps_as_sel.set_active(gtk.TRUE)
    
    # EVENTS
    # Window Manager behaviour
    def e_browse(self, widget):
        filesel = gtk.FileSelection("Select File")
        filesel.set_filename(self.last_path)
        filesel.complete("*.html")
        result = filesel.run()
        if result == gtk.RESPONSE_OK:
            self.file = filesel.get_filename()
            self.entry_html.set_text(self.file)
            self.last_path = os.path.dirname(self.file) + "/" 
        filesel.destroy()
        
    def e_hide(self, widget, event):
        widget.hide()
        # do not destroy this window
        return TRUE

    def e_pressed_ok(self, widget):
        self.get_status()
        self.e_hide(self.window, self.DUMMY)
        self.callback()

    def e_pressed_cancel(self, widget):
        self.e_hide(self.window, self.DUMMY)
    #! EVENTS
