 #
#   File        : winsearch.py
#   Author      : Alain TAUCH <alain.tauch@thalesatm.com>
#   Description : the WinSearch class
#
#   Copyright (C) 2003-2006 Alain TAUCH
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   * Thu Jun 08 2006 Alain TAUCH <alain.tauch@thalesatm.com>
#   - delete_event mishandle fix
#


import gtk
import gobject
from gtk import TRUE, FALSE

DUMMY           = 1

KEY_RETURN      = 65293
KEY_ENTER       = 65421
KEY_ESC         = 65307

class WinSearch:

    def __init__(self, callback):
        self.callback = callback
        self.init_members()
        self.create_window()
        self.init_iface()

    def init_members(self):
        self.exact = FALSE
        self.case = FALSE
        self.txt = ""        

    def init_iface(self):
        self.en_txt.set_text(self.txt)
        self.cb_exact.set_active(self.exact)
        self.cb_case.set_active(self.case)
        self.en_txt.grab_focus()

    def create_window(self):
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        # set window properties
        self.window.set_border_width(8)
        self.window.set_title("RPM Analyzer - Search")
        self.window.set_resizable(FALSE)
        self.window.set_modal(FALSE)
        # connect events
        self.window.connect("delete_event", self.e_hide)
        #
        # VBox for all components 
        vbox_main = gtk.VBox()
        vbox_main.set_spacing(4)
        self.window.add(vbox_main)
        # search frame
        frame_main = gtk.Frame("Search")
        vbox_main.add(frame_main)
        # frame content
        vbox_frame = gtk.VBox()
        vbox_frame.set_border_width(5)
        frame_main.add(vbox_frame)
        # text entry
        self.en_txt = gtk.Entry(30)
        self.en_txt.connect("key-press-event", self.e_keypressed)
        vbox_frame.add(self.en_txt)
        # potars
        self.cb_exact = gtk.CheckButton("exact match")
        self.cb_exact.connect("toggled", self.e_toggle_exact)
        self.cb_case = gtk.CheckButton("case sensitive")
        self.cb_case.connect("toggled", self.e_toggle_case)
        hbox_frame = gtk.HBox()
        hbox_frame.set_spacing(4)
        hbox_frame.add(self.cb_exact)
        hbox_frame.add(self.cb_case)
        vbox_frame.add(hbox_frame)
        # buttons
        hbox_buttons = gtk.HBox()
        hbox_buttons.set_spacing(2)
        find = gtk.Button(stock="gtk-find")
        find.connect("clicked", self.e_find)
        close = gtk.Button(stock="gtk-close")
        close.connect("clicked", self.e_close)
        hbox_buttons.add(find)
        hbox_buttons.add(close)
        vbox_main.add(hbox_buttons)

    def show_all(self):
        self.en_txt.grab_focus()
        self.window.show_all()
            
    # EVENTS
    def e_hide(self, widget, event):
        widget.hide()
        # do not destroy this window
        return TRUE

    def e_find(self, event):
        self.txt = self.en_txt.get_text()
        if self.txt != "":
            self.callback(self.txt)

    def e_close(self, event):
        self.window.hide()

    def e_keypressed(self, widget, event):
        if event.keyval in (KEY_ENTER, KEY_RETURN):
            self.e_find(DUMMY)
        if event.keyval == KEY_ESC:
            self.en_txt.set_text(self.txt)

    def e_toggle_case(self, widget):
        self.case = self.cb_case.get_active()

    def e_toggle_exact(self, widget):
        self.exact = self.cb_exact.get_active()            
    #! EVENTS
